// Selector de fecha
//
// Este widget esta pensado para ser utilizado dentro de nire por lo que
// hace algunas supociones sobre el entorno en que se ejecuta.
//
// Por ejemplo, supone, que la fecha acutal se obtiene por medio de la
// función currentDate y que retornará un objeto momentjs
//
$.widget("nire.dateselector", {
	options: {
		// Fecha mínima que se permite seleccionar.
		minDate: undefined,

		// Fecha máxima que se permite seleccionar.
		maxDate: undefined,

		// Fecha actualmente seleccionada.
		date: undefined,
		
		// Fecha previamente seleccionada.
		from_date: undefined,

		// Funcion a llamar para formatear cada fecha.
		dateFormatter: function(date) { return getDayFormatedv2(date.format("YYYYMMDD")) },
		
		// Funcion a llamar para formatear cada fecha.
		fn_date_clicked: undefined
	},

	//
	// Obtener o setear la fecha actualmente seleccionada
	//
	date: function(date) {
		//console.log("datewidget date: "+ this.options.date);
		if(date === undefined) return this.options.date;
		
		var _date = date.clone().hour(0).minute(0).second(0).millisecond(0);
		
		if( this.options.maxDate && this.options.maxDate.isBefore(_date) ) return false;
		if( this.options.minDate && this.options.minDate.isAfter(_date) ) return false;
		this._select(_date);
	},
	
	//
	// Obtener la fecha previamente seleccionada
	//
	fromDate: function() {
		return this.options.from_date;
	},

	//
	// Obtener o Setear la fecha minima que se puede seleccionar
	//
	minDate: function(date) {
		if(date === undefined) return this.options.minDate;
		
		var _date = date.clone().hour(0).minute(0).second(0).millisecond(0);

		if( this.options.date.isBefore(_date) )
			this.options.date = _date.clone();

		this.options.minDate = _date;
		this._select(this.options.date);
	},

	//
	// Obtener o Setear la fecha máxima que se puede seleccionar
	//
	maxDate: function(date) {
		if(date === undefined) return this.options.maxDate;
		
		var _date = date.clone().hour(0).minute(0).second(0).millisecond(0);

		if( this.options.date.isAfter(_date) )
			this.options.date = _date.clone();

		this.options.maxDate = _date;
		this._select(this.options.date);
	},

	//
	// Ir a la fecha siguiente si es posible
	//
	next: function() {
		var target = this.options.date.clone().add(1, "day");
		if(this.options.maxDate && this.options.maxDate.isBefore(target)) return false;

		this._select(target);
	},

	//
	// Ir a la fecha anterior si es posible
	//
	previous: function() {
		var target = this.options.date.clone().subtract(1, "day");
		if(this.options.minDate && this.options.minDate.isAfter(target)) return false;

		this._select(target);
	},

	//
	// Inicializar el widget.
	//
	// Calcula cuantas fechas se deben mostrar y hacer los binding de los 
	// eventos necesarios
	//
	_create: function() {
		var initialDate = this.options.date;
		if(!initialDate) initialDate = currentDate().hour(0).minute(0).second(0).millisecond(0);

		this._select(initialDate);
	},

	//
	// Seleccionar una fecha específica
	//
	_select: function(date) {
		var _this = this

		var datesElements = this.element.find("a");
		var firstDate = date.clone().subtract(Math.floor(datesElements.length/2), "days");

		var nowDate = moment().hour(0).minute(0).second(0).millisecond(0);
		var currentDate = firstDate.clone();
		
		$(datesElements).removeClass("ui-state-disabled ui-btn-active dateselector-current-date");
		
		datesElements.each(function()
		{
			var str_date = _this.options.dateFormatter(currentDate);
			if ( str_date )
				$(this).text(str_date);
			
			if( currentDate.isSame(date) )
				$(this).addClass("ui-btn-active");
			if ( !currentDate.diff(nowDate) )
				$(this).addClass("dateselector-current-date");
				
			if( _this.options.minDate )
				if( currentDate.isBefore(_this.options.minDate) ) $(this).addClass("ui-state-disabled");

			if( _this.options.maxDate )
				if( currentDate.isAfter(_this.options.maxDate) ) $(this).addClass("ui-state-disabled");

			// Necesito esta higer-function para poder guardar dentro del closure el valor
			// actual de la variable currentDate y no el ultimo valor en la iteracion
			var clickHandler = function(date) {
				if (_this.options.fn_date_clicked) {
					return function() {
						_this.options.fn_date_clicked(date);
					}
					
				} else {
					return function() {
						_this._select(date); return false; 
					}
				}
			};
			$(this).off(".dateselector").on("click.dateselector", clickHandler(currentDate));

			currentDate = currentDate.clone().add(1, "day");
		});

		if( !this.options.date || !date.isSame(this.options.date) || !this.from_date ) {
			this.options.from_date = this.options.date;
			this.options.date = date;
			this._trigger("change", null, date);
		}
	}
});