/**
 * Intenta cargar una imagen. 
 *
 * @param image url de la imagen que se intenta cargar.
 * @return Promise al cual se le ejecutará el método done pasándole como primer paráemtro el nombre de la imagen
 * que se logró cargar. O sino se ejecutará el método fail si no se pudo cargar la imagen.
 */
function tryImage(image) {
	var deferred = $.Deferred();
	var element = $("<img />")
		.on("error", function() { console.log("Error al cargar imagen", image); deferred.reject(); })
		.on("load", function() { deferred.resolve(image); });

	element.attr("src", image);
	return deferred.promise();
}

/**
 * Dado un numero y una unidad, formatea el numero para ser mostrado correctamente.
 * Por ejemplo las kilocalorias no llevan decimales, los gramos se truncan en un decimal, etc.
 * Si no se especifica unidad, se usara el formato defualt que es dos decimales.
 *
 * Si el valor especificado como unidad es un numero, se redondea a esa cantidad de decimales.
 */
function formatNumber(number, unit) {
	var specialCases = {
		kcal: 0,
		mg: 0
	};

	var decimals = 2;
	if(specialCases[unit] !== undefined) decimals = specialCases[unit];
	else if ($.isNumeric(unit)) decimals = unit;
	
	var factor = Math.pow(10, decimals);

	return Math.round(number * factor ) / factor;
}

/**
 * Retornar la n-position de la serie de fibonacci
 * Ejemplos: 
 *     fibonacci(1) = 1
 *     fibonacci(2) = 1
 *     fibonacci(3) = 2
 *     fibonacci(4) = 3
 *     fibonacci(5) = 5
 *     fibonacci(15) = 610
 */
var fibonacci = (function(cache){
    return cache = cache || {}, function(n){
        if (cache[n]) return cache[n];
        else return cache[n] = n == 0 ? 0 : n < 0 ? -fibonacci(-n)
            : n <= 2 ? 1 : fibonacci(n-2) + fibonacci(n-1);
    };
})();

/**
 * Handlebars Helpers
 */
if( typeof Handlebars != 'undefined' ) {
    Handlebars.registerHelper("i18n", function(text, data) {
        var key = text;
        if(data.hash.prefix) key = data.hash.prefix + key;

        return i18n(key)
    });
}
