function initiatePushPlugin() {
    // si no existe el plugin salir sin hacer nada
    if(!window.plugins && !window.plugins.pushNotification) {
        console.log("Push not available. Because PushPlugin is missing")
        return;
    }

    // ANTES esta variable era global, pero no se usa en ningun lado.
    // YO *el autor del commit* la hice _privada_ de forma autoritaria, y sigue todo 
    // funcionando bien, pero si algun día hay problemas con las notifiaciones.
    // convendría hacerla global nuevamente, para probar.

    var pushNotification = window.plugins.pushNotification;
    if ( device.platform == 'android' || device.platform == 'Android' || device.platform == "amazon-fireos" ){
        pushNotification.register(
            getLogger("Android Push Register Success"),
            getLogger("Android Push Register Error"),
            {
                "senderID": GOOGLE_SENDER_ID,
                "ecb":"onNotificationAndroid"
            });
    }
    else {
        pushNotification.register(
            saveClientToken,
            getLogger("iOS Push Register Error"),
            {
                "badge":"true",
                "sound":"true",
                "alert":"true",
                "ecb":"onNotificationAPN"
            }
        );
    }
}

/**
 * Guardar el token que identifica al cliente para notificaciones push para ser enviado
 * en cuanto se sepa cual es el usario logueado al cual asociarle este token.
 *
 * El primer parametro es la plataforma, ya que el servidor necesita saber si el dispositivo
 * es iOS o Android poruqe la forma de enviar las notificaciones varía en cada caso.
 */
function saveClientToken(token) {
    console.log("Saving device Token", token);
    if(!token || token.length == 0) {
        console.log("SaveClientToken Error. El token de cliente no puede estar vacio");
    }

    // Guardamos el token del cliente.
    // No es necesario guardar la plataforma por al momento de ser enviado al servidor
    // se obtiene la plataforma por medio del plugin device
    localStorage.setItem("deviceReg_token", token);
}

//
// Si tenemos un token de autologin, hacemos un login oculto al usuario
// y lo enviamos la home
//
function processAutoLogin(token) {
    if(token) {
        var user = localStorage.getItem(CREDENTIALS_USERNAME);
        var doctype = localStorage.getItem(CREDENTIALS_DOCUMENT_TYPE)
        if(user) {
            Nire.connection.initialize(token, user);
            $( "body" ).pagecontainer( "change", "timeline.html" );
        }
    }
}

/**
 * Callback de notificaciones para iOS
 */
function onNotificationAPN(e) {
    console.log("Notification iOS Received", e);
    onNotification({  
        foreground: e.foreground == "1",
        token: e.token
    });

    // en ios tenemos e.alert, e.sound y e.badget
    // en el caso de badge, podemos usar setApplicationIconBadgeNumber(callback, e.badge)
}

/**
 * Callback de notificación para Android
 */
function onNotificationAndroid(e) {
    console.log("Notification Android Received", e);
    switch( e.event )
    {
    case 'registered':
        return saveClientToken(e.regid);

    case 'message':
        return onNotification({  
            foreground: e.foreground,
            token: e.payload.token
        });

    case 'error':
        console.log("Push Notification Error", e)
        break;

    default:
        console.log("Push Notification Unknown event", e)
        break;
    }

    // en Android tenemos e.coldstart para saber si la aplicación
    // fue iniciada o simplemente restaurada de background
}

function onNotification(params) {
    // Si estamos en background - Procesamos un Autologin
    if(!params.foreground)
        processAutoLogin(params.token);
}
