/* CSS */

$( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pageDeviceWearables.css"/>');

/* VARIABLES */

var pageDeviceWearables = {
		
	$page: null,
		
    getData: function()
    {
    	showLoading();

    	Nire.connection.deviceWearables()
    		.done(function(json_response) {
				hideLoading();
				if (json_response.error.code == 0)
				{
				  	pageDeviceWearables.addContent(json_response.data);
				}
				else
					showAlert(json_response.error.message, null, gjson_lang.error, gjson_lang.accept);
    		});
    },

    createHeader: function()
    {
    	var template = Handlebars.compile($("#template-header_device_wearables").html());
        
    	pageDeviceWearables.$page.find( "#app_header" ).remove();
        
        try {
        	$( "#app_header" ).html( template({ title: gjson_lang["device_wearables_title"]}) ).toolbar("refresh").show();
        } catch (e) {
        	$( "#app_header" ).toolbar().show();
        }
     
        $.mobile.resetActivePageHeight();
        
	    showNotificationUpdate();
    },

    addContent: function(json)
    {
    	pageDeviceWearables.createTracks(json.body.wearables);
	},

    createTracks: function(ajson_tracks)
    {
    	var bg_connection, text_connection, synchronize, disabled, opacity;
        var _this = this;

        var template_track = Handlebars.compile($("#template-track_device_wearables").html());
        var gridPositions = ["ui-block-a", "ui-block-b", "ui-block-c"];
        var type_select = null;

        var tracks = [];
        $.each(ajson_tracks, function(i) {

            if(ajson_tracks[i].connected){
                bg_connection = "bg_activity";
                text_connection = gjson_lang["device_wearables_connected"];
                synchronize = true;
            }
            else{
                bg_connection = "bg_auxiliary";
                text_connection = gjson_lang["device_wearables_disconnected"];
                synchronize = false;
            }

            var synchronize_txt = gjson_lang["device_wearables_synchronize"]

            if(!ajson_tracks[i].active){
                disabled = "ui-state-disabled";
                opacity = "track_disabled"
            }
            else{
                disabled = "";
                opacity = "";
            }

            // Estos son los datos que requiere el template de Handlebars
            var datos_item =  {
                id_wearable: "wearable_" + this.id,
                block_class: gridPositions[i % gridPositions.length],
                image: this.image,
                title: this.title,
                description: this.description,
                con_indicator: bg_connection,
                connection_txt: text_connection,
                btn_disabled: disabled,
                track_disabled: opacity,
                active: this.active,
                synchronize_txt: synchronize_txt,
                synchronize: synchronize
            };

            // Generamos los elementos del DOM y le asociamos al información de este elemento
            // para despues poder recuperar toda la información del servidor
            var track = $(template_track(datos_item)).data("info", this)

            // Agregamos el indicador a la lista de indicadores que se mostraran
            tracks.push( track );
        });

        // Borramos los indicadores anteriores y agregamos los nuevos
        $("#device_wearables_tracks").empty().append(tracks).trigger( "updatelayout" );

        tracksImageErrorControl();

    	pageDeviceWearables.adjustSizesOfTracks();

	},

	adjustSizesOfTracks: function()
	{
		var width = pageDeviceWearables.$page.find(".track_body").width();
		if (width) {
			pageDeviceWearables.$page.find(".div_track_image, .div_track_detail").css("height", width );
		}
		//pageDeviceWearables.adjustSizeOfTracksText();
	},

    changeState: function(data)
    {
        var appUrl = data.connected ? data.unsync_url : data.sync_url;

        var backgroundColor = rgb2hex($("#app_header").css("background-color"));
        var titleColor = rgb2hex($("#page_title").css("color"));
        var browser = cordova.ThemeableBrowser.open(appUrl, '_blank', {
            statusbar: {
                color: '#ffffffff'
            },
            toolbar: {
                height: 44,
                color: backgroundColor
            },
            title: {
                color: titleColor,
                staticText: data.title
            },closeButton: {
                image: 'arrow_left',
                imagePressed: 'arrow_left',
                align: 'left',
                event: 'closePressed'
            }
        }).addEventListener(cordova.ThemeableBrowser.EVT_ERR, function(e) {
            console.error(e.message);
        }).addEventListener(cordova.ThemeableBrowser.EVT_WRN, function(e) {
            console.log(e.message);
        });
        var detectAndCloseBrowser = function(event) {
            if(event.url == "https://api.validic.com/authorizationOk") 
            {
                pageDeviceWearables.getData();
                browser.close()
            }
        }

        var closeBrowser = function() {
            browser.removeEventListener('loadstart', detectAndCloseInAppBrowser);
            browser.removeEventListener('exit', closeInAppBrowser);
        }

        browser.addEventListener('loadstart', detectAndCloseBrowser);
        browser.addEventListener('exit', closeBrowser);
    },

    synchronizeApp: function(data)
    {
        Nire.connection.synchronizeApp(data.refresh_url, data.title)
            .done(function(){
                var alertTxt = String.format(gjson_lang["app_sync_requested"], data.title);
                showToast(alertTxt)
            })
            .fail(function() { showToast("Error with wearable sync."); })
    },

    /**
     * Se hizo click sobre un track, dependiendo el estado se debera conectar o sincronizar.
     */
    onWearableClick: function(info) {
        if(info.connected) this.synchronizeApp(info);
        else this.changeState(info);
    }
};

/* EVENTS */
$.event.special.tap.emitTapOnTaphold = false;

$( window ).off( ".device_wearables" )
    .on("resize.device_wearables", function()
    {
    	if(pageDeviceWearables)
    		pageDeviceWearables.adjustSizesOfTracks();
    });

$( document ).off( ".device_wearables" )

    .on( "pagecreate.device_wearables", "#page_device_wearables", function()
    {
    	pageDeviceWearables.$page = $(this);
    })

    .on("pagecontainershow.device_wearables", function(event, ui)
    {
    	if ( ui.toPage.attr("id") == "page_device_wearables" ) {
    		pageDeviceWearables.createHeader();
    		pageDeviceWearables.getData();
    	}
    })
    
    .on( "pagecontainerchange.device_wearables", function(event, ui)
	{
		if ( ui.prevPage.attr("id") == "page_device_wearables" && ui.toPage.attr("id") != "page_device_wearables" 
				&& ui.toPage.attr("id") != "page_app_direct_communicator" ){
			$("#page_device_wearables").trigger( "pageremove" ).remove();
		}
	})

    .on("pageremove.device_wearables", "#page_device_wearables", function()
	{
    	$( window ).off( ".device_wearables" );
    	$( document ).off( ".device_wearables" );
		$( 'link[href="css/pageDeviceWearables.css"]' ).remove();
		pageDeviceWearables = null;
	})
	
	.on("vmouseup.device_wearables", "#page_device_wearables", function(event)
    {
		$(".track_body.track_active").removeClass("track_active");
    })
		
	.on("vmousedown.device_wearables vmouseup.device_wearables", ".track_body", function(event)
    {
    	$(this).addClass("track_active");
    })

    .on("click.device_wearables", ".wearable_track", function()
    {    	
        pageDeviceWearables.onWearableClick($(this).data("info"));
    })

    .on("click.device_wearables", "#btn_pending_notifications:visible", function(event)
	{
		cloneAppHeader(pageDeviceWearables.$page);
		$( "body" ).pagecontainer("change", "appDirectCommunicator.html", { transition:"slide" });
	});