/* VARIABLES */
var pageLogin2 = {
    currentSubpage: "", // Subpagina que se esta mostrando actualmente. Modificar sólo a travez de showSubpage

    /**
     * Inicializar los componentes y bindings de la página
     */
    initPage: function() {
        this.addStyles();
        this.removeAppHeader();
        this.showHideFunctions();
        this.registerBindings();
        this.setUsername();        

        this.tryPreviousLogin();
    },

    removeAppHeader: function(){
        $( "#app_header" ).toolbar();
        $( "#app_header" ).empty();
        $( "#app_header" ).toolbar( "destroy" );
    },

    setUsername: function(){
        var username = localStorage.getItem(CREDENTIALS_USERNAME)
        if(username && username != "undefined" && username != "null"){
            $("input[name=username]").val(username);
            this.checkUsername();
        }
    },

    /**
     * Liberar recursos utilizados por la página
     */
    unloadPage: function() {
        this.unregisterBindings();
        this.removeStyles();
        pageLogin2 = null;
    },

    /**
     * Muestra una de las subpaginas, y oculta las demás
     */
    showSubpage: function(page) {
        var subpages = ["intro", "username", "checkmail", "password"];
        $.each(subpages, function() {
            $("#subpage_" + this).hide();
            $("#page_login2").removeClass("subpage_" + this);
        })

        $("#subpage_" + page.toLowerCase()).show();
        $("#page_login2").addClass("subpage_" + page.toLowerCase());

        // Guardamos la pagina actual en una variable, aunque sería mejor
        // buscarla por la clase activa, el enfoque de la variable es más
        // facil de implementar.
        this.currentSubpage = page.toLowerCase();
        Nire.analytics.page("login_" + this.currentSubpage);
    },

    /**
     * Habilita el botón de continuar cuando el username ingresado
     * sea un username valido
     */
    checkUsername: function() {
        var value = $("input[name=username]").val();
        var state = value != "";

        if(state) {
            var patt = /^[A-Za-z0-9_\-\.@]*$/; 
            state = patt.test(value);
        }
        
        $('#login_continue').attr('disabled', !state);
    },

    /**
     * Habilita el botón de entrar cuando se haya ingresado un password
     */
    checkPassword: function() {
        var value = $("input[name=password]").val();
        var state = value != "";

        $('#login_enter').attr('disabled', !state);
    },

    /**
     * Realiza un intento de login con el usuario y la contraseña ingresadas
     */
    doPasswordLogin: function() {
        var documentType = $("input[name=documentType]").val();
        var username = $("input[name=username]").val();
        var password = $("input[name=password]").val();

        this.sendLoginRequest(documentType, username, password);
    },

    /**
     * Envia al servidor una peticicion de login.
     */
    sendLoginRequest: function(type, username, password) {
        showCover();
        Nire.analytics.event("Login", "Intentado login por usuario y contraseña");
        Nire.connection.login(username, password, type)
            .done( function(data) {
                Nire.analytics.event("Login", "Utilizando usuario y contraseña");
                LoginHelper.processLoginSuccess(type, username, data); 
            })
            .fail( LoginHelper.processLoginError )
            .always( hideCover );
    },

    sendLoginMail: function() {
        showCover();
        var username = $("input[name=username]").val();

        Nire.analytics.event("Login", "Solicitando email de Login");
        Nire.connection.sendEmail( username )
            .done( this.showSubpage.bind(this, 'CHECKMAIL') )
            .fail( LoginHelper.showServerError )
            .always( hideCover );
    },

    /**
     * Muestra u oculta el boton para acceder a la registración de un nuevo usuario
     */
    showHideFunctions: function() {
        if(SIGNUP_URL) $(".login_register_user").show(); 
        else $(".login_register_user").hide();

        if(LOGIN_METHOD == "PASSWORD") $("#login_by_email").hide();
        else $("#login_by_email").show();
    },

    /**
     * Oculta el splashscreen (si existe)
     * Desde que se llama a este método se demoran unos segundos en ocultar el splash
     */
    hideSplashScreen: function() {
        if (navigator && navigator.splashscreen) {
            setTimeout(function() { navigator.splashscreen.hide(); }, 2000);
        }
    },

    /**
     * Registrar los handlers de los botones
     */
    registerBindings: function() {
        // Botones de continuar
        $(document).on("click.login2", "#login_login", function() {
            pageLogin2.showSubpage('USERNAME');
            return false;
        });
        $(document).on("click.login2", "#login_continue", function() {
            var username = $("input[name=username]").val();
            localStorage.setItem(CREDENTIALS_USERNAME, username);
            pageLogin2.showSubpage('PASSWORD');
             return false;
        });
        $(document).on("click.login2", "#login_send_email", function() {
            pageLogin2.sendLoginMail();
            return false;
        });
        $(document).on("click.login2", "#login_enter", function() {
            pageLogin2.doPasswordLogin();
            return false;
        });

        // Boton de volver
        $(document).on("click.login2", ".header > a.ui-btn", function() {
            var target = $(this).data("back");
            pageLogin2.showSubpage(target);
            return false;
        });

        // Boton de registarme
        $(document).on("click.login2", ".login_register_user, .login_register_user a", function() {
            openBrowser(SIGNUP_URL, i18n("register_user"));
            return false;
        });

        // Boton de recordar contraseña
        $(document).on("click.login2", ".login_lost_password, .login_lost_password a", function() {
            openBrowser(RECOVER_PASS_URL, i18n("login_password"));
            return false;
        });

        // Binding de datos
        $(document).on("keyup.login2", "input[name=username]", this.checkUsername.bind(this));
        $(document).on("keyup.login2", "input[name=password]", this.checkPassword.bind(this));
    },

    /**
     * Desregistrar los handlers de los botones
     */
    unregisterBindings: function() {
        console.log("Desregistrando los bindings previamente guardados");
        $(document).off(".login2");
    },

    /**
     * Agrega los estilos CSS de la página
     */
    addStyles: function() {
        console.log("Agregando estilos");
        $( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pageLogin2.css"/>');
    },

    /**
     * Quita los estilos CSS de la página
     */
    removeStyles: function() {
        console.log("Eliminando estilos");
        $( 'link[href="css/pageLogin2.css"]' ).remove();
    },

    /**
     * Intenta acceder al servidor con las ultimas credenciales utilizadas.
     * Si no lo consigue, muestra el Login
     */
    tryPreviousLogin: function() {
        this.showSubpage("");
        Nire.connection.tryPreviousLogin()
            .always( this.hideSplashScreen )
            .fail( this.showSubpage.bind(this, "intro") )
            .done( function(data) {
                Nire.analytics.event( "login", "Login desde AutoLogin" );
                
                var savedDocumentType = localStorage.getItem(CREDENTIALS_DOCUMENT_TYPE);
                var savedUsername = localStorage.getItem(CREDENTIALS_USERNAME);
                LoginHelper.processLoginSuccess(savedDocumentType, savedUsername, data);
            } );
    },

    /**
     * Se llama cuando se preciona el botón de back en un telefono android o similar.
     */
    onBackKeyDown: function() {
        if(this.currentSubpage == "password") this.showSubpage("username");
        else if(this.currentSubpage == "checkmail") this.showSubpage("password");
        else this.showSubpage("intro");
    }
};

/* EVENTS */
$(window).off( ".login2" )
    .on("resize.login2", function() { $("#page_login2").trigger("updatelayout") })
    ;

$( document ).off( ".login2" )
    .on("pagecreate.login2", "#page_login2", pageLogin2.initPage.bind(pageLogin2))
    .on("pageremove.login2", "#page_login2", pageLogin2.unloadPage.bind(pageLogin2))
    ;