/* CSS */

$( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pageNewTrackNutrition2.css"/>');

/* VARIABLES */
var pageNewTrackNutrition2 = {
        
    $page: null,
    animationDuration: 500,
    dateselectorNumItems: 7,
    delayTimer: null,
    timeslots: null,

    selectedPlates: [], // Platos de comida seleccionados
    searchResults: [],  // Resultados de la búsqueda

    /**
     * Dado un id busca el plato entre los resultados de la busqueda o retorna
     * null si el plato no existe.
     */
    findInResults: function(id) 
    { 
        return this.findInCollection(this.searchResults, id); 
    },

    /**
     * Dado un id busca el plato entre los platos seleccionados o retorna
     * null si el plato no existe.
     */
    findInSelected: function(id) 
    { 
        return this.findInCollection(this.selectedPlates, id); 
    },

    /**
     * Realiza la busqueda de posibles platos
     */
    search: function(query) {
        var self = this;

        if(this.delayTimer) clearTimeout(this.delayTimer);
        this.delayTimer = setTimeout(function() {
            Nire.search.plateByName(query).done(function(items) {
                self.searchResults = items;
                self.updateState();
            });
        }, 250);
    },

    /**
     * Seleccionar un plato
     * @param idPlate identificador del plato seleccionado.
     */
    selectPlate: function( idPlate ) {
        console.log("selectPlate");
        var plate = this.findInResults(idPlate);
        if( !this.findInSelected(idPlate) && plate != null) {
            this.selectedPlates.push(plate);
            this.updateState();
            pageNewTrackNutrition2.$page.find(".track_nutrition_search_results").empty();
            pageNewTrackNutrition2.$page.find("#page_new_track_nutrition2_searchbox").val("");
            pageNewTrackNutrition2.$page.find("#new_track_nutrition2_meal_options_container, #new_track_nutrtion2_days_container, #page_new_track_nutrition2_selected, .new_track_nutrition2_bt_footer").show();
            $("#page_title").text(gjson_lang["add_plate"]);
        }

        

    },

    removePlate: function(idPlate) {
        this.selectedPlates = this.selectedPlates.filter(function(item) { return item.id != idPlate })
        this.updateState();
    },

    /**
     * Enviar los platos seleccionados al servidor
     */
    send: function() {
        var self = this;
        var idMeal = parseInt($(".options_container .selected_item").attr("data-option_item"));
        var current_date = $("#new_track_nutrition2_days_actual .dateselector").dateselector("date").format("YYYYMMDD");

        Nire.connection.addNutrition(current_date, idMeal, this.selectedPlates)
            .fail(function() { console.log("Se produjo un error al agregar un plato.", arguments); })
            .done(function() {
                $("#page_timeline").trigger("pageremove");
                $("#page_timeline").remove();
                if ( $.mobile.navigate.history.stack[$.mobile.navigate.history.stack.length-2].pageUrl.match("nutritionWeekly.html") )
                    $("body").pagecontainer( "change", "nutritionWeekly.html" );
                else
                    $("body").pagecontainer( "change", "timeline.html" );
                // TODO: Detectar si fui llamado desde el home de nutricion y volver a ese lugar.
                //$("body").pagecontainer( "change", "timeline.html" );
            })
            .fail(function() { showAlert(gjson_lang.newTrack_nutrition_error, null, gjson_lang.error, gjson_lang.accept); });
    },
 
    /**
     * Actualiza la vista en base al estado del modelo
     */
    updateState: function() {
        // Mostramos todo los platos seleccionados
        var dom = this.templates.selectedPlates(this.selectedPlates);
        pageNewTrackNutrition2.$page.find("#page_new_track_nutrition2_selected").html(dom);

        var dom = "";
        if(pageNewTrackNutrition2.$page.find("#page_new_track_nutrition2_searchbox").val() !== "")
            dom = this.templates.searchResults(this.searchResults);

        if(this.searchResults.length != 0)
            pageNewTrackNutrition2.$page.find(".track_nutrition_search_results").html(dom);
        else
            pageNewTrackNutrition2.$page.find(".track_nutrition_search_results").html( '<div class="search-result"><p>'+gjson_lang.no_results+'</p></div>');

        // El botón de "Done" solo se activa si hay comida y plato
        if( this.selectedPlates.length > 0 ) $("#page_new_track_nutrition2 #new_track_nutrition2_bt_footer").removeClass("ui-state-disabled");
        else $("#page_new_track_nutrition2 #new_track_nutrition2_bt_footer").addClass("ui-state-disabled");
    },

    /**
     * Precompila los tempates de Handlebars para ser utilizados luego
     */
    precompileTemplates: function() {
        this.templates = {
            searchResults: Handlebars.compile( pageNewTrackNutrition2.$page.find("#template-search-results").html() ),
            selectedPlates: Handlebars.compile( pageNewTrackNutrition2.$page.find("#template-selected-plates").html() )
        };
    },

    /**
     * Buscar un plato, segun su ID en una colección de platos
     */
    findInCollection: function(collection, id) {
        var matches = collection.filter(function(item) {return item.id == id});
        if(matches.length == 0) return null;
        return matches[0];
    },

    backPlate: function() {
        pageNewTrackNutrition2.$page.find(".track_nutrition_search_results").empty();
        $("#page_new_track_nutrition2_searchbox").val("");
        $("#page_new_track_nutrition2_searchbox").blur();
        pageNewTrackNutrition2.$page.find("#new_track_nutrition2_meal_options_container, #new_track_nutrtion2_days_container, #page_new_track_nutrition2_selected, .new_track_nutrition2_bt_footer").show();
        $("#page_title").text(gjson_lang["add_plate"]);
    },
    /**
     * Realizo la carga de los tipos de comida (Desayuno, cena...)
     */
    getData : function(day)
    {
        var _this = this;

        showLoading();
        Nire.connection.newNutrition(day)
            .done(function(json_response) {
                hideLoading();
                if (json_response.error.code == 0)
                    pageNewTrackNutrition2.addContent(json_response.data);
                else
                    showAlert(json_response.error.message, null, gjson_lang.error, gjson_lang.accept);

            });
    },

    /**
     * Creamos el header
     */
    createHeader : function()
    {
        var template = Handlebars.compile($("#template-header_new_track_nutrition2").html());
                
        try {
            $( "#app_header" ).html( template({ title: gjson_lang["add_plate"] }) ).toolbar("refresh").show();
        } catch (e) {
            $( "#app_header" ).toolbar().show();
        }
        $("#page_new_track_nutrition2_searchbox").attr("placeholder", gjson_lang["search_plate"]);

        $.mobile.resetActivePageHeight();
    },

    /**
     * Añadimos contenido con el widget de optionSelector. Le pasamos los valores de servicio newNutrition para construir
     el selector de Desayuno, merienda...
     */   

    addContent : function(json)
    {
        // Si no hay most_used ocultamos esa seccion del html
        //wrapper_title: gjson_lang["nutrition_meal_" + title.toLowerCase()].toUpperCase()

        var default_option = null;
        showLoading();
        Nire.connection.timeslots()
            .done(function(json_response) {
                pageNewTrackNutrition2.timeslots = json_response.data.timeslots;
                hideLoading();

                var now = new Date();
                var actual_time = now.getHours()+':'+now.getMinutes();

                $.each(pageNewTrackNutrition2.timeslots, function(i, item) {
                    // Preguntamos solo por MinRange, por si la hora actual no existe en ningún rango. Mostramos la comida mas cercana.
                    if(actual_time >= item.minRange)
                        default_option_aux = item.name; 
                        default_option = default_option_aux.toLowerCase();
                });
            })
            .fail(function(){
                hideLoading();
            });

        pageNewTrackNutrition2.$page.find("#new_track_nutrition2_meal_options").optionselector({
            item: json.body.food_types,
            wrapper_title: "nutrition_meal_",
            default_option: default_option,
            fn_item_clicked: function(obj_new_item) {
                return false;
            }
        });

    },

    initializeNavbarsOverflow: function()
    {
        // Seteamos las fechas de los otros navbar
        var actual_date = $("#new_track_nutrition2_days_actual .dateselector").dateselector("date");
        var actual_minDate = $("#new_track_nutrition2_days_actual .dateselector").dateselector("minDate");
        var actual_maxDate = $("#new_track_nutrition2_days_actual .dateselector").dateselector("maxDate");
        var int_start = Math.ceil(pageNewTrackNutrition2.dateselectorNumItems/2);
        
        for (var i=int_start, len=int_start+(int_start-1); i<=len; i++)
        {

            var date_ant = actual_date.clone().subtract(i, "day");
            
            var str_date_ant_formatted = getDayFormatedv2(date_ant.format("YYYYMMDD"));
            
            $("#new_track_nutrition2_days_ant .dateselector li:nth-child("+ (pageNewTrackNutrition2.dateselectorNumItems - i) +") a").html(str_date_ant_formatted);
            
            if ( date_ant.isBefore(actual_minDate) )
                $("#new_track_nutrition2_days_ant .dateselector li:nth-child("+ (pageNewTrackNutrition2.dateselectorNumItems - i) +") a").addClass("ui-state-disabled");
            else
                $("#new_track_nutrition2_days_ant .dateselector li:nth-child("+ (pageNewTrackNutrition2.dateselectorNumItems - i) +") a").removeClass("ui-state-disabled");
            
            var date_next = actual_date.clone().add(i, "day");
            
            var str_date_next_formatted = getDayFormatedv2(date_next.format("YYYYMMDD"))
            
            $("#new_track_nutrition2_days_next .dateselector li:nth-child("+ (i + 1) +") a").html(str_date_next_formatted);
            
            if ( date_next.isAfter(actual_maxDate) )
                $("#new_track_nutrition2_days_next .dateselector li:nth-child("+ (i + 1) +") a").addClass("ui-state-disabled");
            else
                $("#new_track_nutrition2_days_next .dateselector li:nth-child("+ (i + 1) +") a").removeClass("ui-state-disabled");
        }
    },
    
    resetNavbarsPosition: function(time)
    {
        var int_time = (time == 0) ? 0 : time || pageNewTrackNutrition2.animationDuration;
        var int_lag = ($( window ).width()/pageNewTrackNutrition2.dateselectorNumItems);
        var int_lagged_items = (Math.floor(pageNewTrackNutrition2.dateselectorNumItems/2));
        var int_lag_total = int_lag * int_lagged_items;
        $("#new_track_nutrition2_days_ant").animate( { "left": -(int_lag_total), "right": int_lag_total }, { duration: int_time, queue: false } );
        $("#new_track_nutrition2_days_actual").animate( { "left": 0, "right": 0 }, { duration: int_time, queue: false } );
        $("#new_track_nutrition2_days_next").animate( { "left": int_lag_total +"px", "right": -(int_lag_total) +"px" }, { duration: int_time, queue: false } );
    },
    
    animateNavbars: function(direction, diff_days)
    {
        var obj_what_date = (diff_days) ? $("#new_track_nutrition2_days_actual .dateselector").dateselector("date").clone().add(diff_days, "day") : null;
        var int_num_days = Math.abs(diff_days);
        var int_lag = ($( window ).width()/pageNewTrackNutrition2.dateselectorNumItems);
        var int_lagged_items = Math.floor(pageNewTrackNutrition2.dateselectorNumItems/2);

        if (direction == "right") {
            var int_left_direction = 1;
            var int_right_direction = -1;
        }
        else if ( direction == "left") {
            var int_left_direction = -1;
            var int_right_direction = 1;
        }
        
        $("#new_track_nutrition2_days_actual").animate( 
            { 
                "left": int_left_direction * int_lag * int_num_days, 
                "right": int_right_direction * int_lag * int_num_days
            }, 
            { duration: pageNewTrackNutrition2.animationDuration, queue: false } 
        );
        
        $("#new_track_nutrition2_days_ant").animate( 
            { 
                "left": -(int_lagged_items * int_lag) + (int_left_direction * int_num_days * int_lag), 
                "right": (int_lagged_items * int_lag) + (int_right_direction * int_num_days * int_lag)
            }, 
            { duration: pageNewTrackNutrition2.animationDuration, queue: false } 
        );
        
        $("#new_track_nutrition2_days_next").animate( 
            { 
                "left": (int_lagged_items * int_lag) + (int_left_direction * int_num_days * int_lag), 
                "right": -(int_lagged_items * int_lag) + (int_right_direction * int_num_days * int_lag)
            }, 
            { duration: pageNewTrackNutrition2.animationDuration, queue: false } 
        )
        .promise()
        .done(function(){
            if ( !$("#new_track_nutrition2_days_actual .dateselector").dateselector("date", obj_what_date) )
                pageNewTrackNutrition2.resetNavbarsPosition();
        });
    },

};

/* EVENTS */

$( window ).off( ".new_track_nutrition2" )
    .on("resize.new_track_nutrition2", function()
    {
        if(pageNewTrackNutrition2) {
            adjustSizeOfGridFooterActions($("#page_new_track_nutrition2"));
        }
    });

$( document ).off( ".new_track_nutrition2" )

    .on( "pagebeforecreate.new_track_nutrition2", "#page_new_track_nutrition2", function()
    {
        $("#page_new_track_nutrition2 #new_track_nutrition2_bt_footer").addClass("ui-state-disabled");
    })
    
    .on("pagecreate.new_track_nutrition2", "#page_new_track_nutrition2", function()
    {
        pageNewTrackNutrition2.$page = $(this);
        $( "#app_header" ).toolbar();      
        pageNewTrackNutrition2.precompileTemplates(); 
    })

    .on("pageshow.new_track_nutrition2", "#page_new_track_nutrition2", function()
    {
        pageNewTrackNutrition2.createHeader();
        adjustSizeOfGridFooterActions($(this));
        pageNewTrackNutrition2.getData( currentDate() );
        
        var minDate = currentDate().subtract(15, "days");
        if(minDate.isBefore(Nire.rangeData.minDate)){
            //si por alguna razon el minDate del usuario es el dia siguiente,
            //fuerzo la fecha actual como minima
            if(Nire.rangeData.minDate.isAfter(currentDate()))
                minDate = currentDate();
            else
                minDate = Nire.rangeData.minDate;
        }

        $("#new_track_nutrition2_days_actual .dateselector").dateselector({
            minDate: minDate,
            maxDate: currentDate(),
            change: function(event, newdate) {
                //pageNewTrackNutrition2.$page.find(".fdata").text(getDateFormatted(newdate.format("YYYYMMDD")));
                pageNewTrackNutrition2.resetNavbarsPosition(0);
                pageNewTrackNutrition2.initializeNavbarsOverflow();
            },
            fn_date_clicked: function(obj_new_date) {
                var obj_from_date = $("#new_track_nutrition2_days_actual .dateselector").dateselector( "date" );
                var int_diff_days = obj_new_date.diff(obj_from_date, 'days');
                if ( obj_new_date.isAfter(obj_from_date) ){
                    pageNewTrackNutrition2.animateNavbars("left", int_diff_days);
                } else if ( obj_new_date.isBefore(obj_from_date) ){
                    pageNewTrackNutrition2.animateNavbars("right", int_diff_days);
                }
                return false;
            }
        });

    })

    .on("focus.new_track_nutrition2", "#page_new_track_nutrition2_searchbox", function()
        { 
            pageNewTrackNutrition2.$page.find("#new_track_nutrition2_meal_options_container, #new_track_nutrtion2_days_container, #page_new_track_nutrition2_selected, .new_track_nutrition2_bt_footer").hide();
            $("#page_title").text(gjson_lang["search_plate"]);
        })

    .on("keyup.new_track_nutrition2", "#page_new_track_nutrition2_searchbox", function() { pageNewTrackNutrition2.search( $(this).val() ); })
    .on("click.new_track_nutrition2", "#page_new_track_nutrition2 #new_track_nutrition2_bt_footer", function() { pageNewTrackNutrition2.send(); })
    .on("click.new_track_nutrition2", ".search-result", function() { pageNewTrackNutrition2.selectPlate($(this).data("id")); })
    .on("click.new_track_nutrition2", "#page_new_track_nutrition2_selected .delete", function() { pageNewTrackNutrition2.removePlate($(this).data("id")); })
     /**
     * Control para el botón back. Si estamos en pantalla principal back de jqm, sino volvemos a mostrar los selectores de comidas
     */
    .on("click.new_track_nutrition2", "#btn_back", function(event) 
    { 
        if ( $("#new_track_nutrition2_meal_options_container").css("display") == "none" )
        {
            event.preventDefault();  
            pageNewTrackNutrition2.backPlate();
            return false;           
        }
        else
            return true;

    })
    


    .on("pageremove.new_track_nutrition2", "#page_new_track_nutrition2", function()
    {
        $( window ).off( ".new_track_nutrition2" );
        $( document ).off( ".new_track_nutrition2" );
        $( 'link[href="css/pageNewTrackNutrition2.css"]' ).remove();
        pageNewTrackNutrition2 = null;
    })
