/* CSS */

$( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pageNutritionWeekly.css"/>');

/* VARIABLES */

var pageNutritionWeekly = {
		
	$page: null,
	animationDuration: 500,
	dateselectorNumItems: 7,
	chart2: null,
	indicatorsWarningMargin: 20,
	
    getData: function(date)
    {
    	showLoading();
    	
    	Nire.connection.nutritionWeekly(date)
    		.done(function(json_response) {
				if (json_response.error.code == 0)
				{
				  	pageNutritionWeekly.addContent(json_response.data, date);
				}
				else
					showAlert(json_response.error.message, null, gjson_lang.error, gjson_lang.accept);
    		})
    		.always(function(){
    			hideLoading();
    		});
    },

    createHeader: function()
    {
    	var template = Handlebars.compile($("#template-header_nutrition_weekly").html())
        
        pageNutritionWeekly.$page.find( "#app_header" ).remove();
        
        try {
        	$( "#app_header" ).html( template({ title: gjson_lang["nutrition_weekly_title"] }) ).toolbar("refresh").show();
        } catch (e) {
        	$( "#app_header" ).toolbar().show();
        }
     
        $.mobile.resetActivePageHeight();
        
	    showNotificationUpdate();
    },

    addContent: function(data, date)
    {
    	if (data.body.title != null)
    		$(".div_indicators2_general_title h3").text(data.body.title);
    	
    	pageNutritionWeekly.prepareDietInfoPopup(data.body);
    	
        pageNutritionWeekly.setquantitiesDone(data.body.indicators, data.body.quantity, data.body.unit, data.body.percent, data.body.totalCalories, data.body.warningMargin);

    	pageNutritionWeekly.createTracks2(data.body.tracks);
        
    	if (!pageNutritionWeekly.prepareTracksForAnimation()) {
    		pageNutritionWeekly.resetNavbarsAndTracksPosition(0);
    	} else {
    		pageNutritionWeekly.resetNavbarsAndTracksPosition(null, 2, function(){
    			$.mobile.silentScroll(0);
    			setTimeout(function(){
                    var img_url = "img/backgrounds/bg_nutrition_" + date.isoWeekday() + ".jpg";
                    var img_tag = new Image();
                    img_tag.onload = function() {
                        $( "#nutrition_weekly_img_background" ).css("background-image", "url("+ img_url +")").trigger("updatelayout");
                        $(this).remove();
                    };
                    img_tag.src = img_url;
    			}, 500);
    		});
    	}
	},
	
	prepareDietInfoPopup: function(json)
	{
		if (json.description == null)
    	{
    		$("#nutrition_weekly_menu_info_btn").css("display", "none");
    	}
    	else
    	{
    		$("#nutrition_weekly_menu_info_btn").jqmData("nire", 
    				{
    					title: json.title, 
    					subtitle: Math.round(formatNumber(json.totalCalories, json.unit)) +" "+ json.unit.capitalize(),
    					description: json.description
    				});
    	}
	},

	createIndicators: function(ajson_indicators, warningMargin)
	{		
        var template_indicator = Handlebars.compile($("#template-indicators_nutrition_weekly").html());

        var indicators = [];

        $.each(ajson_indicators, function(i) {
            // Estos son los datos que requiere el template de Handlebars
//        	console.log("quantity: "+ this.quantity);
//        	console.log("precent: "+ this.precent);
//        	console.log("quantity: "+ this.quantity);
        	var int_completed = Math.round(this.completed);
        	var int_total = Math.round(this.total);
        	
        	var b_alert = ( pageNutritionWeekly.showIndicatorAlert(int_completed, int_total, warningMargin, this.overrideWarningTo) ) ? "color_txt_alert" : null;
        	
            var datos_item =  {
//            	svg_circle_prop: int_svg_circle_prop,
            	svg_circle_class: "svg_circle_"+ this.title,
            	alert_class: b_alert,
            	percentage_completed: int_completed +"%",
            	percentage_total: int_total +"%",
                title: gjson_lang["nutrition_" + this.title]
            };

            // Generamos los elementos del DOM y le asociamos al información de este elemento
            // para despues poder recuperar toda la información del servidor
            var indicator = $(template_indicator(datos_item)).jqmData("nire", this);

            // Agregamos el indicador a la lista de indicadores que se mostraran
            indicators.push( indicator );
        });

        // Borramos los indicadores anteriores y agregamos los nuevos
        $("#page_nutrition_weekly .div_indicators2_general_resume .table_indicators2_wrapper").html(indicators).trigger( "updatelayout" );
                
        var int_svg_circle_prop = Math.round(parseInt($(".indicator2_data_wrapper .indicator2_svg").width())/2);
        
        $(".indicator2_data_wrapper .indicator2_svg circle").attr({ "cx": int_svg_circle_prop, "cy": int_svg_circle_prop, "r": int_svg_circle_prop });
	},
	
	createTracks2: function(ajson_tracks)
	{
		var template_type_wrapper_title = Handlebars.compile($("#template-nutrition_weekly_type_wrapper_title").html());
		var template_type_wrapper_item = Handlebars.compile($("#template-nutrition_weekly_type_wrapper_item").html());

		var date = $("#nutrition_weekly_track_days_actual .dateselector").dateselector("date").format("YYYYMMDD");
		var str_type = null;
        var tracks = [];
                
        $.each(ajson_tracks, function(i, item) {
        	
        	var datos_item = null;
        	
        	if (item.type != str_type)
        	{
        		// Estos son los datos que requiere el template de Handlebars
            	datos_item =  {
            		title: gjson_lang["nutrition_meal_"+ item.type]
            	};
            	// Generamos los elementos del DOM y le asociamos al información de este elemento
                // para despues poder recuperar toda la información del servidor
                var type_wrapper_title = $(template_type_wrapper_title(datos_item));

                // Agregamos el track a la lista de tracks que se mostrarán
                tracks.push( type_wrapper_title );
                
                str_type = item.type;
        	}

            var icon_class = item.sel ? "ui-icon-checked_nutrition" : "ui-icon-unchecked";
            	
            var url_string = "nutritionDetail.html";
            if(isNaN(item.id))
                url_string = "#";
            var str_url_params = "?content="+ encodeURIComponent(this.id) + "&date=" + encodeURIComponent(date) + "&type=" + encodeURIComponent(this.type);
            
            var str_extra_info = "{0} {1} Carb:{2}% Gras:{3}% Prot:{4}%";
            
        	 // Estos son los datos que requiere el template de Handlebars
            datos_item =  {
                url_params: str_url_params,
                title: item.title,
                extra_info: String.format(str_extra_info, formatNumber(item.quantity, item.unit), item.unit.capitalize(), Math.round(item.hc) || 0, Math.round(item.grasa) || 0, Math.round(item.prot) || 0 ),
                icon_class: icon_class,
                url_string: url_string
            };
            // Generamos los elementos del DOM y le asociamos al información de este elemento
            // para despues poder recuperar toda la información del servidor
            var type_wrapper_item = $(template_type_wrapper_item(datos_item)).jqmData("nire", item);
        	
            tracks.push( type_wrapper_item );
        });
        
        // Borramos los tracks anteriores y agregamos los nuevos
       $("#nutrition_weekly_track2_wrapper").html(tracks).listview( "refresh" );
       $("#nutrition_weekly_track2_wrapper li:jqmData(role=list-divider)").prev().addClass("ui-last-child");
	},
	
	drawGraphic: function(percentage)
	{
		if (pageNutritionWeekly.chart2 == null) {
			var chart2 = document.querySelector('#nutrition_weekly_chart2');
			pageNutritionWeekly.chart2 = new EasyPieChart(chart2, {
		        animate: ({ duration: 1000, enabled: true}),
		        scaleLength: 0,
				lineCap: 'square',
				size: parseInt($("#nutrition_weekly_chart2 .p_chart2_data").width()),
		        lineWidth: 4,
		        barColor: $("#nutrition_weekly_chart2").css("color"),
		        onStep: function(from, to, currentValue) {
		            $(this).find(".span_chart2_value").html(Math.round(currentValue));
		        }
		    });
		}
		pageNutritionWeekly.chart2.update(percentage);
	},
	
	drawNutritionChart: function(indicators, warningMargin)
	{
		var $el      = $('#nutrition_weekly_svg_donuts'),
        width   = $el.width(),
        height  = $el.height(),
        r1		= $el.width() * 3/16, // 20
    	r2		= $el.width() * 6/16 , // 30
    	r3		= $el.width() * 6/16 , // 35
    	r4		= $el.width() * 8/16, // 40
    	r_alert	= $el.width() * 2/16; // 10
    
		$el.get(0).setAttribute("viewBox", "0 0 "+ width +" "+ width);
		
		if ( $el[0] )
		{
			$el.empty();
		}
        /* Utils */
        var currier = function(fn) {
            var args = Array.prototype.slice.call(arguments, 1);
            return function() {
                return fn.apply(this, args.concat(
                    Array.prototype.slice.call(arguments, 0)));
            };
        };
        var arcFunctions = [],
            twoPi   = Math.PI * 2;

        var arcFn = function(theArc, d) {
            var interpolate = d3.interpolate(this._current, d);
            this._current = interpolate(0);
            return function(t) {
                return theArc.endAngle(twoPi*interpolate(t))();
            };
        };
        /* Data */
        var dataReal = [Math.round(indicators[0].completed)/100, 
               		 	Math.round(indicators[1].completed)/100, 
               		 	Math.round(indicators[2].completed)/100];
        
        var dataNire = [Math.round(indicators[0].total)/100,
                        Math.round(indicators[1].total)/100,
                        Math.round(indicators[2].total)/100];

        var svg = d3.select('#nutrition_weekly_svg_donuts')
            		.append("g")
            		.attr("transform", "translate(" + width / 2 + "," + height / 2 + ")"),

        meter = svg.append('g').attr('class', 'progress-meter');        

        var arcs = [];
        var start = 0,
            real = 0,
            prev = 0.0;
        var b_alert = false;
        
        for (var i = 0; i < dataNire.length; i++) {
        	// Creamos el grafico de donuts
            if (i>0) {
                start = start + dataNire[i-1];
                real = real + dataReal[i-1];
            }
            arcs[i] = d3.svg.arc()
                .startAngle(real*twoPi)
                .innerRadius(r1)
                .outerRadius(r2);
            arcFunctions[i] = currier(arcFn, arcs[i]);
            meter.append('path').attr('d', d3.svg.arc()
                .startAngle(start*twoPi)
                .endAngle((start + dataNire[i])*twoPi)
                .innerRadius(r3)
                .outerRadius(r4))
                .attr("class", "graph-donut")
                .style("fill", $(".svg_circle_"+ indicators[i].title).css("fill"));
            
            
            // Creamos los indicadores
            d3.select('path.foreground'+i).remove();
            var percentage  = prev,
                foreground  = meter.append('path')
                    .datum(percentage)
                    .attr('class', 'foreground'+i +" graph-donut")
                    .attr('d', function(d) {
                        return arcs[i].endAngle(twoPi*d)(); 
                    })
                    .style("fill", $(".svg_circle_"+ indicators[i].title).css("fill"))
                    .each(function(d) { this._current = d; });

            percentage = prev + dataReal[i];
            meter.select("path.foreground"+i).datum(percentage)
                        .transition().delay(500).duration(1000)
                        .attrTween('d', arcFunctions[i]);
            prev = prev + dataReal[i];
            
            b_alert = pageNutritionWeekly.showIndicatorAlert(dataReal[i], dataNire[i], warningMargin, indicators[i].overrideWarningTo);
        }
        
        if ( b_alert ) {
            setTimeout(function() {
            	meter.append('circle')
	                .attr('circle-anchor', 'middle')
	                .attr('cx', 0)
	                .attr('cy', 0)
	                .attr('r', r_alert)
	                .attr('class', 'graph-alert-circle');
            	meter.append('text')
	                    .attr('text-anchor', 'middle')
	                    .attr('y', r_alert/2)
	                    .attr('class', 'graph-alert-text')
	                    .attr('font-size', width/5)
	                    .text('!');
            }, 1500);
        }

    },
    
    showIndicatorAlert: function(indicatorReal, indicatorRecommended, warningMargin, overrideWarningTo)
    {
    	if ( overrideWarningTo === true )
    		return true;
    	else if ( overrideWarningTo === false )
    		return false;
    	else if ( ( !overrideWarningTo && warningMargin && (indicatorReal - indicatorRecommended > warningMargin) ) 
    			|| ( !warningMargin && (indicatorReal - indicatorRecommended > pageNutritionWeekly.indicatorsWarningMargin) ) )
    		return true;
    	
    	return false;
    },

    /**
     * Marcar un Track como realizado
     */
    updateTrack: function($element)
    {
        var self = this;

        var data = $element.jqmData("nire");
        var date = $("#nutrition_weekly_track_days_actual .dateselector").dateselector("date");

        Nire.analytics.event( "click", "Nutrition track update" );
        vibrate();
        $element.children().last().addClass('rotatingcheck');
        var t0 = Date.now();

        if(data.isUserAdded) {
            Nire.connection.deleteNutritionTrack(date, data.type, data.id)
                .done( function(result) { 
                    self.deleteTrackVisual($element);
                    self.setquantitiesDone(result.data.indicators, result.data.quantity, result.data.unit, result.data.percent, result.data.totalCalories, result.data.warningMargin);
                })
        }
        else {

            Nire.connection.updateTrack(date, data.id, 'nutrition', data.type)
                .done(function(result){
                    pageNutritionWeekly.markTrackCompleted(result, $element);
                    pageNutritionWeekly.setquantitiesDone(result.data.indicators, result.data.quantity, result.data.unit, result.data.percent, result.data.totalCalories, result.data.warningMargin);
                    setTimeout(function() {
                        $element.children().last().removeClass('rotatingcheck');
                    }, (1000 - (Date.now()-t0)));            })
                .fail(function(result) {
                    setTimeout(function() {
                        $element.children().last().removeClass('rotatingcheck');
                    }, 2000);
                });

        }
    },

    /**
     * Quita de la pantalla un track.
     */
    deleteTrackVisual: function($element) {
        $element.slideUp();
    },

    setquantitiesDone: function(indicators, quantityDone, unit, percentage, totalCalories, warningMargin)
    {
    	var int_overhead = null;
    	// reset values
    	$("#nutrition_weekly_progressbar .progress-inner").width(0);
    	$("#nutrition_weekly_progressbar .progress-inner-overhead").width(0);
    	$("#nutrition_weekly_progressbar .progress-text .indicator2_progressbar_consumed").empty();
    	
    	if ( percentage <= 100 )
    	{
    		$("#nutrition_weekly_progressbar .progress-inner").width(Math.round(percentage) +"%");
    	}
    	else
    	{
    		int_overhead = Math.round(percentage) - 100;
    		$("#nutrition_weekly_progressbar .progress-inner").width( (100-int_overhead) +"%");
    		$("#nutrition_weekly_progressbar .progress-inner-overhead").width(int_overhead +"%");
    	}
    	
    	if ( totalCalories )
    	{
    		$("#nutrition_weekly_progressbar .progress-text .indicator2_progressbar_consumed").html(Math.round(formatNumber(quantityDone, unit)));
    		$("#nutrition_weekly_progressbar .progress-text .indicator2_progressbar_recommended").html(Math.round(formatNumber(totalCalories, unit)));
    	}
    	else
    	{
    		var int_consumed_ant = parseInt($("#nutrition_weekly_progressbar .progress-text .indicator2_progressbar_consumed").html()) || 0;
    		$("#nutrition_weekly_progressbar .progress-text .indicator2_progressbar_consumed").html(Math.round(formatNumber(quantityDone, unit)));
    	}	
    	$("#nutrition_weekly_progressbar .progress-text .indicator2_progressbar_unit").html(unit);
        
    	pageNutritionWeekly.createIndicators(indicators, warningMargin);
        
        pageNutritionWeekly.drawNutritionChart(
        		 indicators,
        		 warningMargin
        	);
    },

    markTrackCompleted: function(result, $element)
    {
        var classAdd, classRmv;

        var data = $element.jqmData("nire");

        if(result && result.data && result.data.status == "confirmed"){
            classAdd = 'ui-icon-checked_nutrition';
            classRmv = 'ui-icon-unchecked';
        } else {
            classRmv = 'ui-icon-checked_nutrition';
            classAdd = 'ui-icon-unchecked';
            if (result && result.error && result.error.message == "DATE_AFTER_TODAY") {
            	showToast(gjson_lang.error_nutrition_updatetrack_date_after_today);
            }
        }

        $element.find(".update_track").removeClass(classRmv).addClass(classAdd);
        
        $element.removeClass("track_active");

    },
    
    initializeNavbarsOverflow: function()
    {
    	// Seteamos las fechas de los otros navbar
		var actual_date = $("#nutrition_weekly_track_days_actual .dateselector").dateselector("date");
		var actual_minDate = $("#nutrition_weekly_track_days_actual .dateselector").dateselector("minDate");
		var actual_maxDate = $("#nutrition_weekly_track_days_actual .dateselector").dateselector("maxDate");
    	var int_start = Math.ceil(pageNutritionWeekly.dateselectorNumItems/2);
    	
        for (var i=int_start, len=int_start+(int_start-1); i<=len; i++)
    	{
    		var date_ant = actual_date.clone().subtract(i, "day");
    		
    		var str_date_ant_formatted = getDayFormatedv2(date_ant.format("YYYYMMDD"));
    		
    		$("#nutrition_weekly_track_days_ant .dateselector li:nth-child("+ (pageNutritionWeekly.dateselectorNumItems - i) +") a").html(str_date_ant_formatted);
    		
    		if ( date_ant.isBefore(actual_minDate) )
    			$("#nutrition_weekly_track_days_ant .dateselector li:nth-child("+ (pageNutritionWeekly.dateselectorNumItems - i) +") a").addClass("ui-state-disabled");
    		else
    			$("#nutrition_weekly_track_days_ant .dateselector li:nth-child("+ (pageNutritionWeekly.dateselectorNumItems - i) +") a").removeClass("ui-state-disabled");
    		
    		var date_next = actual_date.clone().add(i, "day");
    		
    		var str_date_next_formatted = getDayFormatedv2(date_next.format("YYYYMMDD"))
    		
    		$("#nutrition_weekly_track_days_next .dateselector li:nth-child("+ (i + 1) +") a").html(str_date_next_formatted);
    		
    		if ( date_next.isAfter(actual_maxDate) )
    			$("#nutrition_weekly_track_days_next .dateselector li:nth-child("+ (i + 1) +") a").addClass("ui-state-disabled");
    		else
    			$("#nutrition_weekly_track_days_next .dateselector li:nth-child("+ (i + 1) +") a").removeClass("ui-state-disabled");
    	}
    },
    
    resetNavbarsAndTracksPosition: function(time, content, successCallback)
	{
		var int_time = (time == 0) ? 0 : time || pageNutritionWeekly.animationDuration;
		var int_lag = ($( window ).width()/pageNutritionWeekly.dateselectorNumItems) * (Math.floor(pageNutritionWeekly.dateselectorNumItems/2));
		if (!content || content == 1) {
    		$("#nutrition_weekly_track_days_ant").animate( { "left": -(int_lag), "right": int_lag }, { duration: int_time, queue: false } );
    		$("#nutrition_weekly_track_days_actual").animate( { "left": 0, "right": 0 }, { duration: int_time, queue: false } );
    		$("#nutrition_weekly_track_days_next").animate( { "left": int_lag, "right": -(int_lag) }, { duration: int_time, queue: false } );
		}
		if (!content || content == 2) {
			$("#nutrition_weekly_animated_content").animate( { "left": 0, "right": 0 }, { duration: int_time, queue: false } )
				.promise()
				.done(function(){
					if (successCallback)
						successCallback();
					$(".dateselector .day_link").removeClass("ui-disabled");
				});
		}
	},
	
	animateNavbars: function(direction, diff_days)
	{
		var int_num_days = Math.abs(diff_days);
		var int_lag = ($( window ).width()/pageNutritionWeekly.dateselectorNumItems);
		var int_lagged_items = Math.floor(pageNutritionWeekly.dateselectorNumItems/2);

		if (direction == "right") {
			var int_left_direction = 1;
			var int_right_direction = -1;
		}
		else if ( direction == "left") {
			var int_left_direction = -1;
			var int_right_direction = 1;
		}
		
		$("#nutrition_weekly_track_days_actual").animate( 
			{ 
				"left": int_left_direction * int_lag * int_num_days, 
				"right": int_right_direction * int_lag * int_num_days
			}, 
			{ duration: pageNutritionWeekly.animationDuration, queue: false } 
		);
		$("#nutrition_weekly_track_days_ant").animate( 
			{ 
				"left": -(int_lagged_items * int_lag) + (int_left_direction * int_num_days * int_lag), 
				"right": (int_lagged_items * int_lag) + (int_right_direction * int_num_days * int_lag)
			}, 
			{ duration: pageNutritionWeekly.animationDuration, queue: false } 
		);
		$("#nutrition_weekly_track_days_next").animate( 
			{ 
				"left": (int_lagged_items * int_lag) + (int_left_direction * int_num_days * int_lag), 
				"right": -(int_lagged_items * int_lag) + (int_right_direction * int_num_days * int_lag)
			}, 
			{ duration: pageNutritionWeekly.animationDuration, queue: false } 
		);
	},
    
    prepareTracksForAnimation: function()
	{
		var int_left = parseInt($("#nutrition_weekly_animated_content").css("left"));
		if (Math.abs(int_left) != 0) {
			$("#nutrition_weekly_animated_content").css( { "left": -(int_left), "right": -(int_left) } );
			return true;
		}
		return false;
	},

	animateTracks: function(direction, diff_days)
	{
		$(".dateselector .day_link").addClass("ui-disabled");
		
		var int_window_width = null;
		var str_what_date = null;

		if (direction == "right") {
			int_window_width = $( window ).width();
			str_what_date = "previous";
		}
		else if ( direction == "left") {
			int_window_width = -($( window ).width());
			str_what_date = "next";
		}
		var obj_what_date = (diff_days) ? $("#nutrition_weekly_track_days_actual .dateselector").dateselector("date").clone().add(diff_days, "day") : null;

		if (int_window_width) {
			$("#nutrition_weekly_animated_content")
				.animate( 
					{ "left": int_window_width, "right": -(int_window_width) },
					{ duration: pageNutritionWeekly.animationDuration, queue: false }
				)
				.promise()
				.done(function(){
					if (obj_what_date) {
						if ( !$("#nutrition_weekly_track_days_actual .dateselector").dateselector("date", obj_what_date)) {
							pageNutritionWeekly.resetNavbarsAndTracksPosition();
						}
					} else {
						if ( !$("#nutrition_weekly_track_days_actual .dateselector").dateselector(str_what_date)) {
							pageNutritionWeekly.resetNavbarsAndTracksPosition();
						}
					}
				})
				.fail(function(){
					pageNutritionWeekly.resetNavbarsAndTracksPosition();
				});
		}
	},
	
    /**
     * Ver el detalle de un track
     */
    viewDetail: function($element)
    {
        var data = $element.jqmData("nire");
        var url = "nutritionDetail.html?content=#id#&date=#date#&type=#type#";
        url = url.replace("#id#", data.id);
        url = url.replace("#type#", data.type);
        url = url.replace("#date#", $("#nutrition_weekly_track_days_actual .dateselector").dateselector("date").format("YYYYMMDD"));

        $( "body" ).pagecontainer( "change", url );
    }
};

/* EVENTS */

$( window ).off( ".nutrition_weekly" )
    .on("resize.nutrition_weekly", function()
    {
    	if(pageNutritionWeekly) {
//    		$('#nutrition_weekly_svg_donuts').css
    		adjustSizeOfGridFooterActions($("#page_nutrition_weekly"));
    	}
    });

$( document ).off( ".nutrition_weekly" )

	.on( "pagecreate.nutrition_weekly", "#page_nutrition_weekly", function()
    {
		pageNutritionWeekly.$page = $(this);
		
		var date = (moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD").isValid()) ? moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD") : moment().hour(0).minute(0).second(0);
		$( "#nutrition_weekly_img_background" ).css("background-image", "url(img/backgrounds/bg_nutrition_" + date.isoWeekday() + ".jpg)").trigger("updatelayout");
		
		var str_start_direction = null;
		
		$( this ).swipe({
			swipeLeft: function()
			{
				pageNutritionWeekly.animateNavbars("left", 1);
				pageNutritionWeekly.animateTracks("left", 1);
			},
			swipeRight: function()
			{
				pageNutritionWeekly.animateNavbars("right", -1);
				pageNutritionWeekly.animateTracks("right", -1);
			},
			swipeStatus: function( event, phase, direction, distance )
			{
//				console.log("str_start_direction: "+str_start_direction);
//				console.log("direction: "+direction);
//				console.log("phase: "+phase);
//				console.log("distance: "+distance);
				if ( phase=="move" )
				{
					if (!str_start_direction)
					{
						str_start_direction = direction;
						if (str_start_direction == "left" || str_start_direction == "right")
							pageNutritionWeekly.$page.swipe("option", "allowPageScroll", "none");
					}
					
					var int_lag = ($( window ).width()/pageNutritionWeekly.dateselectorNumItems) * (Math.floor(pageNutritionWeekly.dateselectorNumItems/2));
					
					if ( direction == "left" && (str_start_direction == "left" || str_start_direction == "right") )
					{
						$("#nutrition_weekly_animated_content").css( { "left": -(distance), "right": distance} );
						$("#nutrition_weekly_track_days_actual").css( { "left": -(distance/pageNutritionWeekly.dateselectorNumItems), "right": distance/pageNutritionWeekly.dateselectorNumItems} );
						$("#nutrition_weekly_track_days_ant").css( { "left": -(int_lag + distance/pageNutritionWeekly.dateselectorNumItems), "right": int_lag + distance/pageNutritionWeekly.dateselectorNumItems } );
						$("#nutrition_weekly_track_days_next").css( { "left": int_lag - distance/pageNutritionWeekly.dateselectorNumItems, "right": -(int_lag - distance/pageNutritionWeekly.dateselectorNumItems) } );
					}
					else if ( direction == "right" && (str_start_direction == "left" || str_start_direction == "right"))
					{
						$("#nutrition_weekly_animated_content").css( { "left": distance, "right": -distance } );
						$("#nutrition_weekly_track_days_actual").css( { "left": distance/pageNutritionWeekly.dateselectorNumItems, "right": -distance/pageNutritionWeekly.dateselectorNumItems } );
						$("#nutrition_weekly_track_days_ant").css( { "left": -(int_lag - distance/pageNutritionWeekly.dateselectorNumItems), "right": int_lag - distance/pageNutritionWeekly.dateselectorNumItems } );
						$("#nutrition_weekly_track_days_next").css( { "left": int_lag + distance/pageNutritionWeekly.dateselectorNumItems, "right": -(int_lag + distance/pageNutritionWeekly.dateselectorNumItems) } );
					}
					else
					{
						return false;
					}
				}
				else if ( phase == "cancel" )
				{
					str_start_direction = null;
					pageNutritionWeekly.$page.swipe("option", "allowPageScroll", "vertical");
					if ( direction != null )
						pageNutritionWeekly.resetNavbarsAndTracksPosition();
				}
				else if ( phase == "end" )
				{
					pageNutritionWeekly.$page.swipe("option", "allowPageScroll", "vertical");
					if (direction != "left" && direction != "right") {
						pageNutritionWeekly.resetNavbarsAndTracksPosition();
					}
					str_start_direction = null;
				}
			},
			allowPageScroll: "vertical",
			cancelThreshold: $( window ).width()/8,
			threshold: $( window ).width()/4,
			excludedElements: ".noSwipe"
		});
    })

    .on("pagecontainershow.nutrition_weekly", function(event, ui)
    {
    	if (ui.toPage.attr("id") == "page_nutrition_weekly") {
	    	pageNutritionWeekly.createHeader();
	    	adjustSizeOfGridFooterActions($("#page_nutrition_weekly"));
    	}
    })
    
    .on( "pagecontainerchange.nutrition_weekly", function(event, ui)
	{
    	if (ui.toPage.attr("id") == "page_nutrition_weekly") {
    		
    		menuHome.getData().done(function(){
	    		$("#nutrition_weekly_track_days_actual .dateselector").dateselector({
	    			date: (moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD").isValid()) ? moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD") : null,
	    			from_date: null,
		            minDate: Nire.rangeData.minDate.clone().hour(0).minute(0).second(0).millisecond(0),
		            maxDate: Nire.rangeData.maxDate.clone().hour(0).minute(0).second(0).millisecond(0),
		            change: function(event, newdate) {
		            	Nire.localStorage.setItem(APP_DATE_STATE, newdate.format("YYYYMMDD"));
		            	pageNutritionWeekly.resetNavbarsAndTracksPosition(0,1);
		            	pageNutritionWeekly.initializeNavbarsOverflow();
		            	pageNutritionWeekly.getData(newdate);
		            },
		            fn_date_clicked: function(obj_new_date) {
			    		var obj_from_date = $("#nutrition_weekly_track_days_actual .dateselector").dateselector( "date" );
			    		var int_diff_days = obj_new_date.diff(obj_from_date, 'days');
			    		if ( obj_new_date.isAfter(obj_from_date) ){
			        		pageNutritionWeekly.animateNavbars("left", int_diff_days);
			    			pageNutritionWeekly.animateTracks("left", int_diff_days);
			        	} else if ( obj_new_date.isBefore(obj_from_date) ){
			        		pageNutritionWeekly.animateNavbars("right", int_diff_days);
			    			pageNutritionWeekly.animateTracks("right", int_diff_days);
			        	}
			    		return false;
			    	}
		        });
    		});
    	} else if ( ui.prevPage.attr("id") == "page_nutrition_weekly" && ui.toPage.attr("id") != "page_nutrition_weekly" 
				&& ui.toPage.attr("id") != "page_app_direct_communicator" ) {
    		
			$("#page_nutrition_weekly").trigger( "pageremove" );
			$("#page_nutrition_weekly").remove();
		}
	})

    .on("pageremove.nutrition_weekly", "#page_nutrition_weekly", function()
	{
    	$( window ).off( ".nutrition_weekly" );
    	$( document ).off( ".nutrition_weekly" );
		$( 'link[href="css/pageNutritionWeekly.css"]' ).remove();
		pageNutritionWeekly = null;
	})

	.on("click.nutrition_weekly", "#nutrition_weekly_menu_info_btn", function()
	{
		event.preventDefault();

		var template_diet_info_popup = Handlebars.compile($("#template-nutrition_weekly_diet_info_popup").html());

		var data = $(this).jqmData("nire");

		var datos_popup = {
				title: data.title,
				subtitle: data.subtitle,
				description: data.description
			};

		pageNutritionWeekly.$page.find(".ui-content").append(template_diet_info_popup(datos_popup));

		$('#nutrition_weekly_diet_info_popup div:jqmData(role=header)').toolbar();
    	$('#nutrition_weekly_diet_info_popup').popup().popup("open");
    	
    	return false;
	})
	
    .on("click.nutrition_weekly", ".update_track", function(event)
    {
    	event.preventDefault();
    	
        pageNutritionWeekly.updateTrack($(this).parent(".track2_item"));
        
        return false;
    })
    
    .on("click.nutrition_weekly", "#btn_pending_notifications:visible", function()
	{
		cloneAppHeader(pageNutritionWeekly.$page);
		$( "body" ).pagecontainer("change", "appDirectCommunicator.html", { transition:"slide" });
	})
	
	.on( "popupbeforeposition.nutrition_weekly", ".ui-popup", function()
    {
    	$("body").css("overflow", "hidden");
    	var maxWidth = $( window ).width() - 50 + "px";
    	var maxHeight = $( window ).height() - 100 + "px";
		$( this ).css({ "max-height": maxHeight, "max-width": maxWidth });
		pageNutritionWeekly.$page.swipe("disable");
    })

    .on( "popupafterclose.nutrition_weekly", ".ui-popup", function()
    {
    	$("body").css("overflow", "auto");
        $( this ).remove();
		pageNutritionWeekly.$page.swipe("enable");
    });
