/* CSS */

$( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pagePillBox.css"/>');

/* VARIABLES */

var pagePillBox = {
	
	$page: null,
	animationDuration: 500,
	dateselectorNumItems: 7,
	aMedicines : [],

    getData: function( date)
    {
    	showLoading();
        var offset = new Date().getTimezoneOffset();
    	Nire.connection.pillBox(date, offset)
    		.done(function(json_response) {
				hideLoading();
				if (json_response.error.code == 0)
				{
					pagePillBox.addContent(json_response.data, date);
				}
				else
					showAlert(json_response.error.message, null, gjson_lang.error, gjson_lang.accept);
    		});
    },

    createHeader: function()
    {
    	var template = Handlebars.compile($("#template-header_pillbox").html())
        
        pagePillBox.$page.find( "#app_header" ).remove();
        
        try {
        	$( "#app_header" ).html( template({ title: gjson_lang["pillbox_title"] }) ).toolbar("refresh").show();
        } catch (e) {
        	$( "#app_header" ).toolbar().show();
        }
     
        $.mobile.resetActivePageHeight();
        
	    showNotificationUpdate();
    },

    addContent: function(data, date)
    {
    	pagePillBox.resetEmptyContent();
    	
    	if (data.rules.length > 0) {
    		pagePillBox.createBody(data);
    	} else {
    		pagePillBox.setEmptyContent();
    	}
    	    	
    	if (!pagePillBox.prepareTracksForAnimation()) {
    		pagePillBox.resetNavbarsAndTracksPosition(0);
    	} else {
    		pagePillBox.resetNavbarsAndTracksPosition(null, 2, function(){
    			$.mobile.silentScroll(0);
    		});
    	}
    	
		setTimeout(function(){
			var img_url = "img/backgrounds/bg_treatments_" + date.isoWeekday() + ".jpg";
            var img_tag = new Image();
            img_tag.onload = function() {
                $( "#pillbox_img_background" ).css("background-image", "url("+ img_url +")").trigger("updatelayout");
                $("#page_pillbox > .ui-footer").show();
                $(this).remove();
            };
            img_tag.src = img_url;			
		}, 500);
	},
	
	resetEmptyContent: function()
	{
		$("#pillbox_empty_content").css("display", "none");
		$("#pillbox_track2_wrapper").show().trigger( "updatelayout" );
	},
	
	setEmptyContent: function()
	{
		//$("#pillbox_data_content").css("display", "none");
		$("#pillbox_track2_wrapper").css("display", "none");
		$("#pillbox_empty_content").show().trigger( "updatelayout" );
		
		var $current_content = pagePillBox.$page.find(".ui-content");
		var int_content_height = getContentHeightToFillScreen(
									$("#app_header"),
									$current_content,
									pagePillBox.$page.find(".ui-footer")
								);
		
		var int_animated_margins = parseInt($("#pillbox_animated_content").css("margin-top")) + parseInt($("#pillbox_animated_content").css("margin-bottom"))
									+ parseInt($("#pillbox_animated_content").css("padding-top")) + parseInt($("#pillbox_animated_content").css("padding-bottom"));
		
		$("#pillbox_empty_content img")
			.one("load", function(){
				
				$("#pillbox_empty_content .centered_in_empty_content")
				.css("margin-top", Math.round( (int_content_height - int_animated_margins - $("#pillbox_track_days_actual").outerHeight() - $("#pillbox_empty_content .centered_in_empty_content").outerHeight())/2) +"px");
			})
			.each(function(){
				if(this.complete)
					$(this).trigger("load");
			});
	},

	createBody: function(ajson_tracks)
	{
		var _this = this;
		var template_type_wrapper_title = Handlebars.compile($("#template-pillbox_type_wrapper_title").html());
		var template_type_wrapper_item = Handlebars.compile($("#template-pillbox_type_wrapper_item").html());

		var date = $("#pillbox_days_actual .dateselector").dateselector("date").format("YYYYMMDD");
		var str_type = null;
        var tracks = [];

        $.each(ajson_tracks.rules, function(i, item) {
        	var datos_item = null;
        	if (item.type != str_type)
        	{
        		// Estos son los datos que requiere el template de Handlebars
            	datos_item =  {
            		title: gjson_lang["nutrition_meal_"+ item.type.toLowerCase()]
            	};
            	
            	// Generamos los elementos del DOM y le asociamos al información de este elemento
                // para despues poder recuperar toda la información del servidor
                var type_wrapper_title = $(template_type_wrapper_title(datos_item));

                // Agregamos el track a la lista de tracks que se mostrarán
                tracks.push( type_wrapper_title );
                
                str_type = item.type;
        	}
        	
            // Estos son los datos que requiere el template de Handlebars
        	//console.log(item.medicines);
            $.each(item.medicines, function(i, item2) {

            	var icon_class = item2.completed ? "ui-icon-checked_treatment" : "ui-icon-unchecked";
            	console.log(item2.title);
            	datos_item =  {
	                title: item2.title,
	                icon_class: icon_class,
	                id: item2.id, 
	                time: item2.time,
	                dosage: item2.dosage,
	        		completed: item2.completed,
	        		description: item2.description,
	        		treatment: item2.treatment,
	        		presentation: item2.presentation  
	            };
            	
            	var type_wrapper_item = $(template_type_wrapper_item(datos_item)).jqmData("nire", item2);
            	tracks.push( type_wrapper_item );	
            });
        });

     	// Borramos los tracks anteriores y agregamos los nuevos
        $("#pillbox_track2_wrapper").html(tracks).listview( "refresh" );
        $("#pillbox_track2_wrapper li:jqmData(role=list-divider)").prev().addClass("ui-last-child");
        
        pagePillBox.adjustSizesOfGrids();
	},
   
    initializeNavbarsOverflow: function()
    {
    	// Seteamos las fechas de los otros navbar
    	var actual_date = $("#pillbox_days_actual .dateselector").dateselector("date");
		var actual_minDate = $("#pillbox_days_actual .dateselector").dateselector("minDate");
		var actual_maxDate = $("#pillbox_days_actual .dateselector").dateselector("maxDate");
		var int_start = Math.ceil(pagePillBox.dateselectorNumItems/2);
		
        for (var i=int_start, len=int_start+(int_start-1); i<=len; i++)
    	{
    		var date_ant = actual_date.clone().subtract(i, "day");
    		
    		var str_date_ant_formatted = getDayFormatedv2(date_ant.format("YYYYMMDD"));
    		
    		$("#pillbox_days_ant .dateselector li:nth-child("+ (pagePillBox.dateselectorNumItems - i) +") a").html(str_date_ant_formatted);
    		
    		if ( date_ant.isBefore(actual_minDate) )
    			$("#pillbox_days_ant .dateselector li:nth-child("+ (pagePillBox.dateselectorNumItems - i) +") a").addClass("ui-state-disabled");
    		else
    			$("#pillbox_days_ant .dateselector li:nth-child("+ (pagePillBox.dateselectorNumItems - i) +") a").removeClass("ui-state-disabled");
    		
    		var date_next = actual_date.clone().add(i, "day");
    		
    		var str_date_next_formatted = getDayFormatedv2(date_next.format("YYYYMMDD"))
    		
    		$("#pillbox_days_next .dateselector li:nth-child("+ (i + 1) +") a").html(str_date_next_formatted);
    		
    		if ( date_next.isAfter(actual_maxDate) )
    			$("#pillbox_days_next .dateselector li:nth-child("+ (i + 1) +") a").addClass("ui-state-disabled");
    		else
    			$("#pillbox_days_next .dateselector li:nth-child("+ (i + 1) +") a").removeClass("ui-state-disabled");
    	}
        console.log($("#pillbox_days_next .dateselector").height());
        $("#pillbox_days_ant .dateselector").css("height", $("#pillbox_days_next .dateselector").height());
        console.log($("#pillbox_days_next .dateselector").height());
        console.log($("#pillbox_days_ant .dateselector").height());
    },
    
    resetNavbarsAndTracksPosition: function(time, content)
	{
    	var int_time = (time == 0) ? 0 : time || pagePillBox.animationDuration;
		var int_lag = ($( window ).width()/pagePillBox.dateselectorNumItems) * (Math.floor(pagePillBox.dateselectorNumItems/2));
		if (!content || content == 1) {
			$("#pillbox_days_ant").animate( { "left": -(int_lag), "right": int_lag }, { duration: int_time, queue: false } );
			$("#pillbox_days_actual").animate( { "left": 0, "right": 0 }, { duration: int_time, queue: false } );
    		$("#pillbox_days_next").animate( { "left": int_lag, "right": -(int_lag) }, { duration: int_time, queue: false } );
		}
		if (!content || content == 2) {
			$("#pillbox_animated_content").animate( { "left": 0, "right": 0 }, { duration: int_time, queue: false } );
		}
	},
	
	animateNavbars: function(direction, diff_days)
	{
		var int_num_days = Math.abs(diff_days);
		var int_lag = ($( window ).width()/pagePillBox.dateselectorNumItems);
		var int_lagged_items = Math.floor(pagePillBox.dateselectorNumItems/2);

		if (direction == "right") {
			var int_left_direction = 1;
			var int_right_direction = -1;
		}
		else if ( direction == "left") {
			var int_left_direction = -1;
			var int_right_direction = 1;
		}
		
		$("#pillbox_days_actual").animate( 
			{ 
				"left": int_left_direction * int_lag * int_num_days, 
				"right": int_right_direction * int_lag * int_num_days
			}, 
			{ duration: pagePillBox.animationDuration, queue: false } 
		);
		$("#pillbox_days_ant").animate( 
			{ 
				"left": -(int_lagged_items * int_lag) + (int_left_direction * int_num_days * int_lag), 
				"right": (int_lagged_items * int_lag) + (int_right_direction * int_num_days * int_lag)
			}, 
			{ duration: pagePillBox.animationDuration, queue: false } 
		);
		$("#pillbox_days_next").animate( 
			{ 
				"left": (int_lagged_items * int_lag) + (int_left_direction * int_num_days * int_lag), 
				"right": -(int_lagged_items * int_lag) + (int_right_direction * int_num_days * int_lag)
			}, 
			{ duration: pagePillBox.animationDuration, queue: false } 
		);
	},
	
    updatePillBox: function($element)
    {
    	var data = $element.jqmData("nire");        
        var date = data.fullTime;
        vibrate();
        $element.children().last().addClass('rotatingcheck');
        var t0 = Date.now();
        console.log(data);
        console.log(date, data.id, 'treatment', data.completed);
        Nire.connection.updateTrack(date, data.id, 'treatment', data.completed)
            .done(function(result){
            	pagePillBox.markPillBoxCompleted(result, $element);
            	setTimeout(function() {
            		$element.children().last().removeClass('rotatingcheck');
            	}, (1000 - (Date.now()-t0)));
       		})
       		.fail(function(result) {
       			setTimeout(function() {
       				$element.children().last().removeClass('rotatingcheck');
       			}, 2000);
       		});
    },
    
    markPillBoxCompleted: function(result, $element)
    {
        var classAdd, classRmv;

        var data = $element.jqmData("nire");

        if(result && result.data && result.data.status == "confirmed"){
            classAdd = 'ui-icon-checked_treatment';
            classRmv = 'ui-icon-unchecked';
        } else {
            classRmv = 'ui-icon-checked_treatment';
            classAdd = 'ui-icon-unchecked';
            if (result && result.error && result.error.message == "DATE_AFTER_TODAY") {
            	showToast(gjson_lang.error_treatment_updatetrack_date_after_today);
            }
        }

        $element.find(".update_track").removeClass(classRmv).addClass(classAdd);
        
        $element.removeClass("track_active");

    },

	prepareTracksForAnimation: function()
	{
		var int_left = parseInt($("#pillbox_animated_content").css("left"));
		if (Math.abs(int_left) != 0) {
			$("#pillbox_animated_content").css( { "left": -(int_left), "right": -(int_left) } );
			return true;
		}
		return false;
	},

	animateTracks: function(direction, diff_days)
	{
		var int_window_width = null;
		var str_what_date = null;

		if (direction == "right") {
			int_window_width = $( window ).width();
			str_what_date = "previous";
		}
		else if ( direction == "left") {
			int_window_width = -($( window ).width());
			str_what_date = "next";
			
		}
		var obj_what_date = (diff_days) ? $("#pillbox_days_actual .dateselector").dateselector("date").clone().add(diff_days, "day") : null;

		if (int_window_width) {
			$("#pillbox_animated_content")
				.animate(
					{ "left": int_window_width, "right": -(int_window_width) },
					{ duration: pagePillBox.animationDuration, queue: false }
				)
				.promise()
				.done(function(){
					if (obj_what_date) {
						if ( !$("#pillbox_days_actual .dateselector").dateselector("date", obj_what_date)) {
							pagePillBox.resetNavbarsAndTracksPosition();
						}
					} else {
						if ( !$("#pillbox_days_actual .dateselector").dateselector(str_what_date)) {
							pagePillBox.resetNavbarsAndTracksPosition();
						}
					}
				})
				.fail(function(){
					pagePillBox.resetNavbarsAndTracksPosition();
				});
		}
	},  

    adjustSizesOfGrids: function()
	{
		var widthB = pagePillBox.$page.find(".body_content .ui-block-b").width();
		var widthContent = pagePillBox.$page.find(".body_content").width();
		if (widthB && widthContent)
			pagePillBox.$page.find(".body_content .ui-block-a").css("width", widthContent - widthB );
	},
     
};

/* EVENTS */

$( window ).off(".pillbox")
    .on("resize.pillbox", function()
    {
    	if(pagePillBox) {
    		pagePillBox.adjustSizesOfGrids();
    		adjustSizeOfGridFooterActions($("#page_pillbox"));
    		if ($("#pillbox_empty_content").is(":visible"))
    			pagePillBox.setEmptyContent();
    	}
    });

$(document).off( ".pillbox" )
    
    .on("pagecreate.pillbox", "#page_pillbox", function()
	{
    	pagePillBox.$page = $(this);
    	
    	var date = (moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD").isValid()) ? moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD") : moment().hour(0).minute(0).second(0);
		$( "#pillbox_img_background" ).css("background-image", "url(img/backgrounds/bg_treatments_" + date.isoWeekday() + ".jpg)").trigger("updatelayout");

    	var str_start_direction = null;
    	
		$( this ).swipe({
			swipeLeft: function() {
				pagePillBox.animateNavbars("left", 1);
				pagePillBox.animateTracks("left", 1);
			},
			swipeRight: function() {
				pagePillBox.animateNavbars("right", -1);
				pagePillBox.animateTracks("right", -1);
			},
			swipeStatus: function( event, phase, direction, distance ) {
				
				if ( phase=="move" ) {
					
					if (!str_start_direction) {
						str_start_direction = direction;
						if (str_start_direction == "left" || str_start_direction == "right")
							pagePillBox.$page.swipe("option", "allowPageScroll", "none");
					}
					
					var int_lag = ($( window ).width()/pagePillBox.dateselectorNumItems) * (Math.floor(pagePillBox.dateselectorNumItems/2));
					
					if ( direction == "left" && (str_start_direction == "left" || str_start_direction == "right") ) {
						$("#pillbox_animated_content").css( { "left": -(distance), "right": distance} );
						$("#pillbox_days_actual").css( { "left": -(distance/pagePillBox.dateselectorNumItems), "right": distance/pagePillBox.dateselectorNumItems} );
						$("#pillbox_days_ant").css( { "left": -(int_lag + distance/pagePillBox.dateselectorNumItems), "right": int_lag + distance/pagePillBox.dateselectorNumItems } );
						$("#pillbox_days_next").css( { "left": int_lag - distance/pagePillBox.dateselectorNumItems, "right": -(int_lag - distance/pagePillBox.dateselectorNumItems) } );
					} else if ( direction == "right" && (str_start_direction == "left" || str_start_direction == "right") ) {
						$("#pillbox_animated_content").css( { "left": distance, "right": -distance } );
						$("#pillbox_days_actual").css( { "left": distance/pagePillBox.dateselectorNumItems, "right": -distance/pagePillBox.dateselectorNumItems } );
						$("#pillbox_days_ant").css( { "left": -(int_lag - distance/pagePillBox.dateselectorNumItems), "right": int_lag - distance/pagePillBox.dateselectorNumItems } );
						$("#pillbox_days_next").css( { "left": int_lag + distance/pagePillBox.dateselectorNumItems, "right": -(int_lag + distance/pagePillBox.dateselectorNumItems) } );
					} else {
						return false;
					}
				} else if ( phase == "cancel" ) {
					
					str_start_direction = null;
					pagePillBox.$page.swipe("option", "allowPageScroll", "vertical");
					pagePillBox.resetNavbarsAndTracksPosition();
					
				} else if ( phase == "end" ) {

					pagePillBox.$page.swipe("option", "allowPageScroll", "vertical");
					if (direction != "left" && direction != "right") {
						pagePillBox.resetNavbarsAndTracksPosition();
					}
					str_start_direction = null;
				}
			},
			allowPageScroll: "vertical",
			cancelThreshold: $( window ).width()/8,
			threshold: $( window ).width()/4,
			excludedElements: ".noSwipe"
		});
	})

    .on("pagecontainershow.pillbox", function(event, ui)
    {
    	if (ui.toPage.attr("id") == "page_pillbox") {
	    	pagePillBox.createHeader();
	    	adjustSizeOfGridFooterActions($("#page_pillbox"));
    	}
    })
    
    .on( "pagecontainerchange.pillbox", function(event, ui)
	{
    	if (ui.toPage.attr("id") == "page_pillbox") {
    		
    		menuHome.getData().done(function(){
	    		$("#pillbox_days_actual .dateselector").dateselector({
	    			date: (moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD").isValid()) ? moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD") : null,
	    			from_date: null,
		            minDate: Nire.rangeData.minDate.clone().hour(0).minute(0).second(0).millisecond(0),
		            maxDate: Nire.rangeData.maxDate.clone().hour(0).minute(0).second(0).millisecond(0),
		            change: function(event, newdate) {
		            	Nire.localStorage.setItem(APP_DATE_STATE, newdate.clone().format("YYYYMMDD"));
		            	pagePillBox.resetNavbarsAndTracksPosition(0,1);
		            	pagePillBox.initializeNavbarsOverflow();
		            	pagePillBox.getData(newdate);
		            },
		            fn_date_clicked: function(obj_new_date) {
			    		var obj_from_date = $("#pillbox_days_actual .dateselector").dateselector( "date" );
			    		var int_diff_days = obj_new_date.diff(obj_from_date, 'days');
			    		if ( obj_new_date.isAfter(obj_from_date) ){
			        		pagePillBox.animateNavbars("left", int_diff_days);
			    			pagePillBox.animateTracks("left", int_diff_days);
			        	} else if ( obj_new_date.isBefore(obj_from_date) ){
			        		pagePillBox.animateNavbars("right", int_diff_days);
			    			pagePillBox.animateTracks("right", int_diff_days);
			        	}
			    		return false;
			    	}
		        });
    		});
    		pagePillBox.adjustSizesOfGrids();
    		
    	} else if ( ui.prevPage.attr("id") == "page_pillbox" && ui.toPage.attr("id") != "page_pillbox" 
				&& ui.toPage.attr("id") != "page_app_direct_communicator" ) {
    		
			$("#page_pillbox").trigger( "pageremove" );
			$("#page_pillbox").remove();
		}
	})

    .on("pageremove.pillbox", "#page_pillbox", function()
	{
    	$(window).off( ".pillbox" );
    	$(document).off( ".pillbox" );
		$( 'link[href="css/pagePillBox.css"]' ).remove();
		pagePillBox = null;
	})
	
	.on("click.pillbox", ".enter_detail", function(event)
	{
		event.preventDefault();
		
		var template_pillbox_popup = Handlebars.compile($("#template-pillbox_popup").html());

		var data = $(this).parent().jqmData("nire");
		console.log(data);
		var datos_popup = {
				treatment: data.treatment,
				title: data.title,
				presentation: data.presentation,
				dosage: data.dosage,
				description: data.description
			};

		pagePillBox.$page.find(".ui-content").append(template_pillbox_popup(datos_popup));

		$('#pillbox_popup div:jqmData(role=header)').toolbar();
    	$('#pillbox_popup').popup().popup("open");
    	return false;
	})
	
	.on("click.pillbox", ".update_track", function()
	{
		pagePillBox.updatePillBox($(this).parents(".track2_item"));
	})
	
	.on( "popupbeforeposition.pillbox", "#pillBox_description", function()
    {
    	$("body").css("overflow", "hidden");
    	var maxWidth = $( window ).width() - 50 + "px";
		var maxHeight = $( window ).height() - 100 + "px";
		$( this ).css({ "max-height": maxHeight, "max-width": maxWidth });
    })

	.on("popupafterclose.pillbox", "#pillBox_description", function()
	{
		$("body").css("overflow", "auto");
	})	
	
    .on("click.pillbox", "#btn_pending_notifications:visible", function()
	{		
		cloneAppHeader(pagePillBox.$page);
		$( "body" ).pagecontainer("change", "appDirectCommunicator.html", { transition:"slide" });
	});
