var pageProfileQuestion = {
    /**
     * Solicita al servidor las preguntas e inicializa con ellas
     * el formulario de preguntas de perfil
     */
    loadQuestions: function() {
        var endpoint = (getURLParameter("full") ?
            Nire.connection.getPreguntas : 
            Nire.connection.getPreguntasPendientes
        ).bind(Nire.connection);

        endpoint().done(function(questions) {
            this.profileQuestions.load(questions);
            this.profileQuestions.showNextQuestion();
        }.bind(this));
    },

    /**
     * Guardar los cambios de la pregunta actual y mostrar
     * la siguiente pregunta. Si no hay más preguntas ir al Timeline
     */
    saveAndNextQuestion: function() {
        if(this.profileQuestions.currentQuestion) {
            var value = this.profileQuestions.currentQuestion.value;
            Nire.connection.respuesta(this.profileQuestions.currentQuestion.id, value);
        }

        var question = this.profileQuestions.showNextQuestion();
        if(!question) {
            $( "body" ).pagecontainer( "change", "timeline.html" );
        }
    },

    /**
     * Inicializar los componentes y bindings de la página
     */
    initPage: function(event, ui) {
        this.addStyles();
        this.registerBindings();

        Nire.connection.initialize();

        // Configuramos e inicializamos el modulo de preguntas
        var params = {
            onShowCallbacks : [ this.onQuestionShow.bind(this) ],
            onChangeCallbacks: [ this.onQuestionChange.bind(this) ],
            templates: {
                option: Handlebars.compile($("#template-options").html()),
                checkbox: Handlebars.compile($("#template-checkbox").html()),
                elasticsearch_result: Handlebars.compile($("#template-elasticsearch-result").html()),
                elasticsearch_selection: Handlebars.compile($('#template-elasticsearch-selection').html())
            }
        }
        this.profileQuestions = new Nire.ProfileQuestions(Nire.search, params);
    },

    /**
     * Comenzar a trabajar sobre la página
     */
    startPage: function(event, ui) {
        if (ui.toPage.attr("id") == "page_profile_question")
        {
            $( "#app_header" ).toolbar();
            $( "#app_header" ).empty();
            $( "#app_header" ).toolbar( "destroy" );

            adjustSizeOfGridFooterActions($("#page_profile_question"));    

            this.loadQuestions();
        }
    },

    /**
     * Se muestra (o redibuja) una pregunta. Utilizamos este evento para
     * inicilizar JQuery Mobile y hacer los ajustes necesarios en el DOM
     */
    onQuestionShow: function() {
        $("#profile_question_formulario").trigger("create");

        if(this.profileQuestions.currentQuestion.mandatory) {
            $("#bt_footer_1").addClass( "ui-state-disabled" );
        }
        else {
            $( "#bt_footer_1" ).removeClass( "ui-state-disabled" );
        }

        // Aprovechamos que mostramos la pregunta para setear tambien el boton de siguiente
        this.onQuestionChange();
    },

    /**
     * El valor de una pregunta cambio, utilizamos este evento para validar
     * la respuesta actual y saber si activar el botón de siguiente o no.
     */
    onQuestionChange: function() {
        // Si es una pregunta elasticsearch borrar la busqueda
        // cuando se selecciona algo
        if(this.profileQuestions.currentQuestion.type === "elasticsearch") {
            var searchinput = $(this.profileQuestions.params.bodyPlaceholder).find("input");
            searchinput.val("").trigger("input");
        }

        if(this.profileQuestions.isValid()) {
            $("#btn_siguiente").removeClass( "ui-state-disabled" );
        }
        else {
            $( "#btn_siguiente" ).addClass( "ui-state-disabled" );
        }
    },

    /**
     * Liberar recursos utilizados por la página
     */
    unloadPage: function() {
        this.unregisterBindings();
        this.removeStyles();
    },

    registerBindings: function() {
        $("#profile_question_formulario")
            .on( "submit.profile_question2", function(event){event.preventDefault();return false;} );
        $("#btn_siguiente")
            .on("click.profile_question2", this.saveAndNextQuestion.bind(this) );
    },

    /**
     * Desregistrar los handlers de los botones
     */
    unregisterBindings: function() {
        $( window ).off( ".profile_question2" )
        $( document ).off( ".profile_question2" );
        // Por si acaso con el remove del DOM no se eliminan los listeners...
        $("#profile_question_formulario").off(".profile_question2");
        $("#btn_siguiente").off(".profile_question2");
    },

    /**
     * Agrega los estilos CSS de la página
     */
    addStyles: function() {
        this.removeStyles();
        $( "body" ).append('<link id="__PROFILE_QUESTION_STYLES__" rel="stylesheet" type="text/css" href="css/pageProfileQuestion.css?' + Date.now() + '" />');

        // Imagen de fondo de la página. (aleatoria entre 0 y 6?)
        $( "#profile_question_img_background" )
            .css("background-image", "url(img/backgrounds/" + Math.floor(Math.random() * 7) + ".jpg)").trigger("updatelayout");

    },

    /**
     * Quita los estilos CSS de la página
     */
    removeStyles: function() {
        $("#__PROFILE_QUESTION_STYLES__").remove();
    }
};

/* EVENTS */
$( window ).off( ".profile_question2")

    .on( "resize.profile_question2", function()
    {
        if ( pageProfileQuestion )
        {
            adjustSizeOfGridFooterActions($("#page_profile_question"));
        }
    });

$( document ).off( ".profile_question2" )

    .on("pagecreate.profile_question2", "#page_profile_question", pageProfileQuestion.initPage.bind(pageProfileQuestion))
    .on("pagecontainershow.profile_question2", pageProfileQuestion.startPage.bind(pageProfileQuestion))
    .on("pageremove.profile_question2", "#page_profile_question", pageProfileQuestion.unloadPage.bind(pageProfileQuestion))
    ;