/*CSS*/
$( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pageRegistryBiography.css"/>');

var pageRegistryBiography = {
    firstTime: true,

	createHeader: function()
    {
        $( "#app_header" ).toolbar();
        $( "#app_header" ).empty();
        $( "#app_header" ).toolbar( "destroy" );
    },

    //
    // Cuando estamos accediendo al perfil ya estando registrados
    // o cuando se accede desde una integracion que "precarga" el perfil con
    // información (ejemplo colsanitas) se llama a este método para solicitar
    // la información actual al servidor.
    //
    getData: function()
    {
        showCover();
        Nire.connection.obtainBiography()
            .always(function() { hideCover() })
            .done(function(result) {
                $('#registry_activity option[value='+result.data.registryActivity+']').attr('selected', 'selected');
                $('#registry_activity').selectmenu('refresh', true);

                if(result.data.date)
                    $("#date").val(result.data.date);

                if(result.data.gender) {
                    if(result.data.gender == "man")
                        $("#radio-choice-man").attr("checked",true);                    
                    else
                        $("#radio-choice-woman").attr("checked",true);
                }

                $("input[name='radio-choice-genre']").checkboxradio("refresh");
                $("#height").val(result.data.height);
                $("#weight").val(result.data.weight);

                if(result.data.bloquedData){
                    pageRegistryBiography.disableInputs(result.data.bloquedData)
                }

                pageRegistryBiography.setNextButtonState()
            });
        
    },

    disableInputs: function(bloquedInputs){
        var inputFields = {
            "height": ["#height"],
            "weight": ["#weight"],
            "date": ["#date"],
            "gender": ["#radio-choice-man", "#radio-choice-woman"],
            "activity": ["#registry_activity"]
        }

        bloquedInputs.split(",").forEach(function(item) {
            var toDisable = inputFields[item];
            if(toDisable) {
                toDisable.forEach(function(selector) {
                    $(selector).prop('disabled', true);
                })
            }
        });
    },
    
    confirm: function(event)
    {    	
        var registryActivity = $("#registry_activity").val();
        var date = $("#date").val();
        var gender = $('input[name=radio-choice-genre]:checked').val();
        var height = $("#height").val();
        var weight = $("#weight").val();

        Nire.connection.registryBiography(registryActivity, date, gender, height, weight)
        	.done(function(json_response) {
        		if(json_response.error.code === 0) {
                    var questionPage = "profileQuestion.html";
                    if(!this.firstTime) questionPage += "?full=true"
        			$("body").pagecontainer( "change", questionPage );
        		} else {
        			showAlert(json_response.error.message, null, gjson_lang.error, gjson_lang.accept);
        		}
        	})
        	.fail(function() {
        		showAlert(gjson_lang.registry_biography_error, null, gjson_lang.error, gjson_lang.accept);
        	});
    },

    /**
     * Si los campos obligatorios estan completos y con valores válidos
     * activa el el boton de continuar
     */
    setNextButtonState: function()
    {
        var disableNext = function() { $("#page_registry_biography #bt_footer_1").addClass("ui-state-disabled"); };
        var enableNext = function() { $("#page_registry_biography #bt_footer_1").removeClass("ui-state-disabled"); };

        // Si alguna validacion no es exitosa, salimos deshabilitando el boton de siguiente
        if(!this.isValidAcceptTerms()) return disableNext();
        if(!this.isValidGender()) return disableNext();
        if(!this.isValidAge()) return disableNext();
        if(!this.isValidWeight()) return disableNext();
        if(!this.isValidHeight()) return disableNext();

        return enableNext();
	},

    validateAge: function(){
        if(!this.isValidAge()) showToast(gjson_lang["registry_biography_invaliddate_range"]);
    },

    isValidAge: function() {
        if(!$('#date').prop('disabled')) {
            var MIN_BIRTHDAY = moment().subtract(VALUES_LIMITS.age.max, 'years');
            var MAX_BIRTHDAY = moment().subtract(VALUES_LIMITS.age.min, 'years');
            var birthday = moment($("#date").val());

            if(!birthday.isValid())
                return false;

            return birthday.isBetween(MIN_BIRTHDAY, MAX_BIRTHDAY);
        } return true;
    },

    isValidGender: function() {
        var gender = $('input[name=radio-choice-genre]:checked').val();
        return gender;
    },

    validateWeight: function() {
        if(!this.isValidWeight()) {
            var height = parseInt($("#weight").val());
            if(!$.isNumeric(height)) showToast(gjson_lang["registry_biography_invalidweight"]);
            else showToast(gjson_lang["registry_biography_invalidweight_range"]);
        }
    },

    isValidWeight: function() {
        if(!$('#weight').prop('disabled')) {
            var weight = parseInt($("#weight").val());
            return $.isNumeric(weight) && weight > VALUES_LIMITS.weight.min && weight < VALUES_LIMITS.weight.max;
        } return true;
    },

    validateHeight: function() {
        if(!this.isValidHeight()) {
            var height = parseInt($("#height").val());
            if(!$.isNumeric(height)) showToast(gjson_lang["registry_biography_invalidheight"]);
            else showToast(gjson_lang["registry_biography_invalidheight_range"]);
        }
    },

    isValidHeight: function() {
        if(!$('#height').prop('disabled')) {
            var height = parseInt($("#height").val());
            return $.isNumeric(height) && height > VALUES_LIMITS.height.min && height < VALUES_LIMITS.height.max;
        } return true;
    },

    isValidAcceptTerms: function(){
        var acceptTerms = !this.firstTime || $("#input_accept_terms_of_use").prop('checked');
        return acceptTerms;
    },

    /**
     * Se llama cuando se preciona el botón de back en un telefono android o similar.
     */
    onBackKeyDown: function() {
        // Aca necesitamos un poco de wizardy...
        // Si venimos de un login, y se preciona Back se debe mostrar el msg de salir de la app
        // pero si vengo de otro lado, debería volver al timeline.
        //
        // Si llegamos a este eveto estamos seguro que estamos en Cordova por lo que usamos
        // método de Cordova impunemente.
        var prevPage = $.mobile.activePage.data('ui.prevPage');
        if(prevPage.is("#page_login2")) {
            navigator.notification.confirm(
                    gjson_lang.exit_msg, // message
                    onConfirm, // callback to invoke with index of button pressed
                    gjson_lang.exit_title, // title
                    [gjson_lang.exit_ok, gjson_lang.exit_cancel]          // buttonLabels
                );

        }
        else {
            navigator.app.backHistory();
    }
    }

};

$( document ).off( ".registry_biography" )

	.on("pagebeforecreate.registry_biography", "#page_registry_biography", function()
	{
		$( "#weight" ).attr("placeholder", gjson_lang["weight_placeholder"]);
		$( "#height" ).attr("placeholder", gjson_lang["height_placeholder"]);
		
		if (TERM_OF_USE_URL)
			$( "#a_accept_terms_of_use" ).attr("href", TERM_OF_USE_URL );

		if (Nire.localStorage.getItem(TERMS_OF_USE) == "true") {
			$("#input_accept_terms_of_use").prop('checked', true);
		}

        var template = Handlebars.compile($("#template-activity").html())

        var activity = {
            high: gjson_lang["activity_high"],
            moderate: gjson_lang["activity_moderate"],
            low: gjson_lang["activity_low"],
            none: gjson_lang["activity_none"]
        }

        $( "#registry_activity" ).html( template(activity) );
	})
	
    .on("pagecontainerbeforeshow.registry_biography", function(event, ui)
    {
        if (ui.toPage.attr("id") == "page_registry_biography")
        {
            pageRegistryBiography.createHeader();

            pageRegistryBiography.getData();
        
            if(getURLParameter("full")){
                pageRegistryBiography.firstTime = false;
                $("#table_accept_terms_of_use").hide()
            }
        }
    })// 

	.on("pageremove.registry_biography", "#page_registry_biography", function()
	{
		$( document ).off( ".registry_biography" );
		$( 'link[href="css/pageRegistryBiography.css"]' ).remove();
		pageRegistryBiography = null;
	})

	.on("change.registry_biography", "#input_accept_terms_of_use", function()
	{
		Nire.localStorage.setItem(TERMS_OF_USE, $("#input_accept_terms_of_use").prop('checked'));
	})

    .on("blur.registry_biography", "#height", function()
    {
        pageRegistryBiography.validateHeight();
    })

    .on("blur.registry_biography", "#weight", function()
    {
        pageRegistryBiography.validateWeight();
    })

    .on("blur.registry_biography", "#date", function(){
        pageRegistryBiography.validateAge();
    })

    // Cuando se escribe cualquier cosa, o se hace click
    // o se realiza un OnBlur verificamos si se debe
    // cambiar el estado del boton siguiente.
    .on("keyup.registry_biography click.registry_biography change.registry_biography", function() {
        pageRegistryBiography.setNextButtonState();
    })

    .on("click.registry_biography", "#bt_footer_1", pageRegistryBiography.confirm);
