/* CSS */
$( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pageTimeline.css"/>');
$( "body" ).append('<link rel="stylesheet" type="text/css" href="css/pageCoverHome.css"/>');

/* VARIABLES */

var pageTimeline = {

	$page: null,
	$previousWrapper: null,
	$previousStarter: null,
	timeslots: null,
	pendingGetData: null,
	retries: 0,
    newUser: false,
    previousWrapperStarter: false,
	animationDuration: 500,
	notificationId: null,
	state: {
		isPreviousWrapperShowed: false
	},

	cancelGetData: function()
	{
    	// Si habia un getData calendarizado lo cancelamos
    	clearTimeout( pageTimeline.pendingGetData );
    	pageTimeline.pendingGetData = null;
	},

	timedGetData: function(timeout, date)
	{
		pageTimeline.cancelGetData();
		pageTimeline.retries = pageTimeline.retries + 1;
        pageTimeline.addCover();
		pageTimeline.pendingGetData = setTimeout( function() {
			pageTimeline.getData(date);
		}, timeout);

		// si se intentó más 5 veces recuperar una dieta
		// y aun no hay info, enviamos una solicitud de regeneracion
		if(pageTimeline.retries == 5) {
			pageTimeline.retries = 0;
			pageTimeline.generarDieta();
		}
	},

    getData: function(date)
    {
    	pageTimeline.cancelGetData();

    	showLoading();

        var offset = new Date().getTimezoneOffset();

        if(!pageTimeline.timeslots){
        	Nire.connection.timeslots()
    			.done(function(json_response) {
    				if (json_response.error.code == 0 && json_response.data.timeslots.length > 0)
    				{
    					pageTimeline.timeslots = json_response.data.timeslots;
                        pageTimeline.getTimelineData(date, offset);
    				}
    				else {
                        var message = json_response.error.message;
                        if(json_response.error.code == 0) {
                            message = "No hay timeslosts"
                        }
    					showAlert(message, null, gjson_lang.error, gjson_lang.accept);
                    }
    			})
    			.fail(function(){
    				hideLoading();
    			});
        } else
            pageTimeline.getTimelineData(date, offset);
    },

    getTimelineData: function(date, offset)
    {
        Nire.connection.timeline(date, offset)
            .done(function(json_response)
            {
                if (json_response.error.code == 0)
                {
//                              try {
                        pageTimeline.addContent(json_response.data, date);
//                              } catch(e){}
                }
                else {
                    // Si el servidor informa que se esta generando una dieta redirigimos a una pantalla
                    // para indicarle al usuario que espere, esa pantalla luego de una pequeña espera
                    // deberia reintentar.
                    if(json_response.error.code == 1)
                    {
                        showLoading();
                        try {
                            pageTimeline.timedGetData(5000, date);
                        } catch(e){}
                    }
                }

                if($("#cover")[0]){
                    pageTimeline.emptyCover();
                }
            })
            .fail(function()
            {
                pageTimeline.timedGetData(5000, date);
            })
            .always(function()
            {
                hideLoading();
//                          var screenWidth = (window.screen.width * window.devicePixelRatio);
//                          var screenhHeight = (window.screen.height * window.devicePixelRatio);
//                          Nire.connection.appHelp(screenWidth, screenhHeight, calculateDPI())
//                              .done(function(json_response) {
//                                  try {
//                                      pageTimeline.checkHelp(json_response.data);
//                                  } catch(e){}
//                              })
//                              .always(function(){
//                                  hideLoading();
//                              });
            });
    },

    generarDieta: function()
    {    	
        Nire.connection.generarDieta();
    },

    createHeader: function()
    {
    	var template = Handlebars.compile($("#template-header_timeline").html());

    	pageTimeline.$page.find( "#app_header" ).remove();

        try {
        	$( "#app_header" ).html( template() ).toolbar("refresh").show();
        } catch (e) {
        	$( "#app_header" ).toolbar().show();
        }

        $.mobile.resetActivePageHeight();

	    showNotificationUpdate();
    },

    addContent: function(data, date)
    {
    	$("#timeline_track_days_actual a:nth-child(2)").text( formatHeaderDate(date, moment().hour(0).minute(0).second(0).millisecond(0)) );
    	$("#timeline_nutrition_previous_wrapper, #timeline_nutrition_actual_wrapper, #timeline_nutrition_next_wrapper, #timeline_physical_activity_wrapper").empty();
    	$(".timeline_add_on_notification, .timeline_add_on_image").remove();

    	var obj_timeline_sections = pageTimeline.getNutritionTimeslotsWrappers(data.body, date);

    	if ( obj_timeline_sections[0].length > 0 )
    	{
    		pageTimeline.prepareNutritionPreviousWrapper();
    		pageTimeline.createTracksTimeslot("#timeline_nutrition_previous_wrapper", obj_timeline_sections[0]);
    		$("#timeline_nutrition_previous_wrapper").append("<a href=\"nutritionWeekly.html\" class=\"section_wrapper_action has_text_shadow\">"+ gjson_lang.timeline_section_actual_action +"</a>");
    	}
    	else
    	{
    		$("#timeline_nutrition_previous_wrapper, #timeline_nutrition_previous_starter").hide();
    	}

    	pageTimeline.createLastMeasures(data.body.measures);

    	if ( obj_timeline_sections[1].length > 0 )
    	{
    		pageTimeline.createTracksTimeslot("#timeline_nutrition_actual_wrapper", obj_timeline_sections[1]);
    		$("#timeline_nutrition_actual_wrapper").append("<a href=\"nutritionWeekly.html\" class=\"section_wrapper_action has_text_shadow\">"+ gjson_lang.timeline_section_actual_action +"</a>");
    	}

    	if ( data.body.physicalActivity.length > 0 )
    	{
    		// Añadir e inicializar los componentes jqm creados para actividad física
    		var obj_physicalActivity_section = pageTimeline.getPhysicalActivityWrapper(data.body.physicalActivity);
    		pageTimeline.createTracksTimeslot("#timeline_physical_activity_wrapper", obj_physicalActivity_section);
    		$("#timeline_physical_activity_wrapper").append("<a href=\"physicalActivityWeekly.html\" class=\"section_wrapper_action has_text_shadow\">"+ gjson_lang.timeline_section_physicalActivity_action +"</a>");
    	}

    	if ( obj_timeline_sections[2].length > 0 )
    	{
    		pageTimeline.createTracksTimeslot("#timeline_nutrition_next_wrapper", obj_timeline_sections[2]);
    		$("#timeline_nutrition_next_wrapper").append("<a href=\"nutritionWeekly.html\" class=\"section_wrapper_action has_text_shadow\">"+ gjson_lang.timeline_section_actual_action +"</a>");
    	}

    	pageTimeline.createAddOnWrappers(data.body.add_on, date);

    	pageTimeline.prepareTracksForAnimation();
    	pageTimeline.resetTracksPosition(0);

    	if ( !pageTimeline.state.isPreviousWrapperShowed )
    	{
    		$("#timeline_animated_content").slideDown("slow")
    			.promise()
    			.done(function(){
    				pageTimeline.$previousWrapper.css("top", "-"+ (pageTimeline.$previousWrapper.outerHeight(true) - pageTimeline.$previousStarter.height()) +"px");
    				$.mobile.resetActivePageHeight();
    			});
    	}
    	else
    	{
    		pageTimeline.showNutritionPreviousWrapper();
    		$("#timeline_animated_content").show(0, function(){
    				pageTimeline.$previousWrapper.css("top", "-"+ (pageTimeline.$previousWrapper.outerHeight(true) - pageTimeline.$previousStarter.height()) +"px");
    				$.mobile.resetActivePageHeight();
    			});
    	}
    	
		setTimeout(function(){
            var img_url = "img/backgrounds/" + date.isoWeekday() + ".jpg";
            var img_tag = new Image();
            img_tag.onload = function() {
                $( "#timeline_img_background" ).css("background-image", "url("+ img_url +")").trigger("updatelayout");
                $("#page_timeline > .ui-footer").show();
                $(this).remove();
            };
            img_tag.src = img_url;
		}, 500);
	},

	createTracksTimeslot: function(pattern, html_elements)
	{
		// Añadir e inicializar los componentes jqm creados para nutrición
		$(pattern).html(html_elements).find(".timeline_track_wrapper").listview();
		$(pattern).find(".timeline_track_container").collapsible();
	},

	createLastMeasures: function(ajson_measures)
	{
        var template_measures_last = Handlebars.compile($("#template-timeline_measures_last").html());

        var a_grid_class = ["solo","a","b"];
    	var a_block_class = ["a","b","c"];

        var a_measures = [];

        $.each(ajson_measures, function(i)
        {
        	if (i==3) return;
        	// Agregamos la medida a la lista de medidas que se mostrarán
        	var str_url = null;
        	var str_url_parmas = null;
        	var str_disabled_class = null;
        	var str_quantity = null;
        	var str_unit = null;
        	
        	if ( this.active )
        	{
        		str_url = "indicatorDetail.html";
        		str_url_parmas = "?content="+ this.type;
        		str_quantity = precise_round(this.quantity,2);
        		str_unit = this.unit;
        	}
        	else
        	{
        		str_url = "newTrackHealth.html";
            	str_url_parmas = "?value="+ this.type;
            	str_quantity = '<img src="img/others/ic_nutrition_plus.png" width="40" height="40" />';
            	str_disabled_class = "ui-btn-active";
        	}
        	
            a_measures.push({
            	block_class: "ui-block-"+ a_block_class[i],
            	url: str_url,
            	url_params: str_url_parmas,
            	disabled_class: str_disabled_class,
            	value_class: ( this.alert ) ? "color_txt_danger" : "color_txt_nutrition",
            	quantity: str_quantity,
            	unit: str_unit,
                name: gjson_lang[this.type],
            });
        });
        // Estos son los datos que requiere el template de Handlebars
        var datos_wrapper =  {
            wrapper_title: gjson_lang["timeline_last_measures"].toUpperCase(),
            wrapper_action_text: gjson_lang["timeline_last_measures_action"],
            grid_class: "ui-grid-"+ a_grid_class[(ajson_measures.length-1) % 3],
            last_measures: a_measures
        };        
        // Borramos los indicadores anteriores y agregamos los nuevos
        $("#timeline_measures_last_wrapper").html(template_measures_last(datos_wrapper)).trigger( "updatelayout" );
	},

	createAddOnWrappers: function(ajson_add_on, date)
	{
		var template_add_on_type = null;
		var item_type = null;
		var str_path = null;
		var str_action_text = null;

        if(moment(date).isSame(moment(), 'day')) {
            ajson_add_on = $.grep(ajson_add_on, function(e){ return e.type == "image"; });

            var notifications = Nire.notification.getLastOrUnanswered()
            for(var idx in notifications) {
                ajson_add_on.push(notifications[idx]);
            }            
        }

		$("#timeline_animated_content [class^=timeline_add_on_]").remove();

		$.each(ajson_add_on, function(i, item)
		{
			if ( item.type === "notification" )
			{
				template_add_on_type = Handlebars.compile($("#template-timeline_adds_on_type_notification").html());
				
				if ( item.content.button )
				{
                    if(item.content.action) {
                        str_path = {url: "appDirectCommunicator.html"};
                        str_action_text = gjson_lang.notification_message_response;
                    } else {
    					str_path = pageTimeline.getNotificationActionPath("CONFIRM", item.content.button, item.content.id);
    					str_action_text = item.content.button.split("#")[0];
                    }
				}  else if(item.content.answer_json){
                    var buttonData = JSON.parse(item.content.answer_json.replace(/'/g, "\""));
                    var link = buttonData.link? buttonData.link : buttonData.appLink;
                    str_path = {url: link};
                    if(buttonData.link) str_path.external = true;
                    str_action_text = buttonData.text;
                }
				else if ( item.content.action && item.content.value !== null )
				{
					str_path = pageTimeline.getNotificationActionPath(item.content.action, item.content.value, item.content.id);
					str_action_text = (str_path) ? gjson_lang.notification_message_response : null;
				}
				
		        item_type = {
		        		id: item.content.id,
		        		notification_class: (str_path) ? "notification_wrapper_with_action" : "notification_wrapper",
		        		title: item.content.title.toUpperCase() + " - " + pageTimeline.formatMedicineTime(moment(item.content.date)),
		        		message: item.content.message,
		        		dismiss_text: (str_path) ? gjson_lang["profile_question_footer_1"] : gjson_lang["notification_read"],
		        		action_url: (str_path) ? str_path.url : null,
		        		action_params: (str_path) ? str_path.params : null,
		        		action_text: str_action_text
		        };
                if(str_path && str_path.external) item_type.external = true;
			}
			else if ( item.type === "image" )
			{
				template_add_on_type = Handlebars.compile($("#template-timeline_add_on_type_image").html());

				item_type = {
						content: item.content
		        };
			}

			if ( template_add_on_type && item.priority )
	        {
	        	$("#timeline_measures_last_wrapper").after(template_add_on_type(item_type));
	        }
	        else if ( template_add_on_type )
	        {
	        	$("#timeline_animated_content").append(template_add_on_type(item_type));
	        }
		});

		$(".timeline_add_on_image img").error(function(){
			$(this).parent().remove();
		});
		if (APP_DOMAIN == "vivebien") 
		{
			var sponsor_text = { text: gjson_lang.menu_home_sponsor_text };
			var template_add_sponsor_image = Handlebars.compile($("#template-timeline_sponsor_image").html());
			$("#timeline_animated_content").after(template_add_sponsor_image(sponsor_text));
		}
	},

	getPhysicalActivityWrapper: function(ajson_physicalActivity)
	{
		var timeline_type_wrapper = Handlebars.compile($("#template-timeline_type_wrapper").html());
		var datos_wrapper = {
				type_class: "type_physicalActivity",
            	collapsed: "false",
            	wrapper_title: gjson_lang.timeline_section_physicalActivity_title,
            	action_url: "nutritionWeekly.html",
            	action_text: gjson_lang.timeline_section_physicalActivity_action
    	};

        // Generamos los elementos del DOM
        var type_wrapper = $(timeline_type_wrapper(datos_wrapper));

		type_wrapper.find(".timeline_track_wrapper").append(pageTimeline.getTrackItems(ajson_physicalActivity, "physicalActivity"))

		return type_wrapper;
	},

    /**
     * Generar un array con todas las franjas horarias posibles.
     *
     * Esto es retorna los timeslots conocidos y genera timeslots sin nombre
     * para las demás horas.
     *
     * Nunca se retornarán dos timeslots sin nombres consecutivos.
     * IMPORTANTE: Se supone que los timeslots no se superponen!
     *
     * EJEMPLO: si los timeslots son Desayuno de 9 a 10, Comida de 12 a 15hs
     * el resultado esperado es <unnamed> de 0 a 9, Desayuno de 9 a 10, <unnamed>
     * de 10 a 12, Comida de 12 a 15, <unnamed> de 15 a 24
     */
    getFullTimeslots: function() {
        var time = function(str) { return moment(str, "HH:mm"); }
        var nextTimeslot = function(baseTimeslot) {
            // Agregamos un timeslot al final para competar siempre las 24hs
            var copyTimeslots = $.extend(true, [], this.timeslots);
            copyTimeslots.push({name: "", minRange: "24:00", maxRange: "24:00"});

            return copyTimeslots.reduce(function(previous, current) {
                if(!baseTimeslot && !previous) return current;
                if(baseTimeslot && !time(current.minRange).isAfter(time(baseTimeslot.minRange))) return null;

                if(!previous) return current;
                if(!time(current.minRange).isAfter(time(previous.minRange))) return current;
                return previous;
            }, null);
        }.bind(this)

        var timeslots = []

        var baseTimeslot = null;
        var nextNamedTimeslot = nextTimeslot(null);
        while(nextNamedTimeslot !== null) {

            // Si no hay un timeslot inicial, lo creamos
            if(!baseTimeslot) baseTimeslot = {
                minRange: "00:00",
                maxRange: "00:00",
                name: ""
            }

            // Agregamos timeslot sintetico
            if(baseTimeslot.maxRange !== nextNamedTimeslot.minRange) {
                timeslots.push({
                    minRange: baseTimeslot.maxRange,
                    maxRange: nextNamedTimeslot.minRange,
                    name: ""
                });
            }

            // Agregamos timeslot real (si dura al menos un minuto) y tiene nombre
            // esto es para evitar el de las 24:00 final
            if(nextNamedTimeslot.maxRange !== nextNamedTimeslot.minRange && nextNamedTimeslot.name !== "") {
                timeslots.push(nextNamedTimeslot);
            }

            baseTimeslot = nextNamedTimeslot;
            nextNamedTimeslot = nextTimeslot(baseTimeslot);
        }

        return timeslots;
    },

    getNutritionTimeslotsWrappers: function(json_body,date) {
        // Template del contenedor de cada timeslot
        var timeline_type_wrapper = Handlebars.compile($("#template-timeline_type_wrapper").html());

        // Convertir un string en formato HH:mm a un objeto moment
        var time = function(str) { return moment(str, "HH:mm"); }

        // Crear un wrapper para un timeslot
        var buildWrapper = function(title, haveMedicines, section) {
            // si no estamos en el dia actual, nunca van colapsadas
            var collapsed = moment().isSame(date, "day");

            var datos_wrapper = {
                type_class: "type_nutrition",
                collapsed: collapsed && section == 2 ? "true" : "false",
                wrapper_title: gjson_lang["nutrition_meal_" + title.toLowerCase()].toUpperCase()
            };

            if(haveMedicines && Nire.connection.getEnabledSections().indexOf("TREATMENT") >= 0) {
                datos_wrapper.wrapper_title_icon = "<img src=\"img/others/ic_pill.png\" class='SECTION_TREATMENT' />";
                datos_wrapper.has_icon_class = "has_icon";
            }

            return $(timeline_type_wrapper(datos_wrapper));
        }

        // Elegir en que seccion del timeline aparece un timeslot
        // 0: si ya pasó (y es del dia actual)
        // 1: si estamos en ella actualmente (y es del dia actual)
        // 2: si es futura (o de un dia diferente al actual)
        var selectSection = function (timeslot) {
            // Si el dia al que corresponden los datos no es hoy, siempre van
            // en la seccion del futuro. (la variable date es parte el contexto)
            if(!moment().isSame(date, "day")) return 2;

            // Si timeslot es un string, lo interpretamos como un timeslot de un momento unico
            if(typeof timeslot === "string")
                timeslot = { maxRange: timeslot, minRange: timeslot };

            if (moment().isAfter( time(timeslot.maxRange) )) return 0;
            if (moment().isBefore( time(timeslot.minRange) )) return 2;
            return 1;
        }

        // Obtener una estructura de timeslots completa.
        var timeslots = this.getFullTimeslots()
        
        function findTimeslotsByName (name) {
            return timeslots.reduce(function (previous, current) { 
                if(current.name.toLowerCase() == name.toLowerCase()) return current;
                return previous;
            }, null);
        }
        
        function findTimeslotsByTime (str) {
            return timeslots.reduce(function(previous, current) {
                var between = time(str).isBetween(time(current.minRange), time(current.maxRange));
                between = between || time(str).isSame(time(current.minRange), "minute");
                between = between || time(str).isSame(time(current.maxRange), "minute");
                
                if( between )
                    return current;

                return previous;
            }, null);
        }

        // Agregamos las comidas a los timeslots
        json_body.nutrition.forEach(function (item) {
            var timeslot = findTimeslotsByName(item.type);

            // si no se encontró el timeslot correcto, utilizamos cena.
            if(timeslot === null) {
                getLogger("Timeline")("Se buscó un timeslot que no existe.", item.type);
                timeslot = findTimeslotsByName("cena");
            }

            timeslot.nutrition = item.items;
        });

        // Agregamos las medicinas a los timeslosts
        json_body.medicines.forEach(function (item) {
            item.medicines.forEach(function (medicine) {
                var timeslot = findTimeslotsByTime( item.time );
                if(!timeslot.medicines) timeslot.medicines = [];
                timeslot.medicines.push(medicine);
            });
        });

        var obj_timeline_sections = [[], [], []];
        timeslots.forEach(function (timeslot) {
            // Los timeslots sin nombre solo pueden contener medicinas. Y en estos casos
            // cada medicina se agrega independientemente de las demás.
            if(timeslot.name === "" && timeslot.medicines) {
                timeslot.medicines.forEach(function (medicine) {
                    $.merge(obj_timeline_sections[ selectSection(medicine.time) ], pageTimeline.getTrackItems([medicine], "treatment"));
                });
            }

            // Un timeslot con nombre puede contener medicinas y/o comidas.
            // En estos casos se agregan todos juntos en un grupo
            else if(timeslot.name !== "" && (timeslot.medicines || timeslot.nutrition)) {
                var haveMedicines = !!timeslot.medicines;
    
                // Si el timeslot corresponde al futuro, pero en la seccion del presente
                // no hay datos, entonces forzamos que el timezone sea el presente
                // excepto si estamos en un dia que no es el actual
                var section = selectSection(timeslot);
                if(moment().isSame(date, "day"))
                    if(section == 2 && obj_timeline_sections[1].length == 0) section = 1; 

                var wrapper = buildWrapper( timeslot.name, haveMedicines, section );
                wrapper.find(".timeline_track_wrapper")
                    .append(pageTimeline.getTrackItems(timeslot.nutrition || [], "nutrition", timeslot.name))
                    .append(pageTimeline.getTrackItems(timeslot.medicines || [], "treatment", timeslot.name));

                obj_timeline_sections[ section ].push( wrapper );
            }
        });

        return obj_timeline_sections;
    },

	/**
	 * Obtener los grupos de tracks solicitados según la franja horaria
	 */
	getNutritionTimeslotsWrappers_OBSOLETE: function(json_body, date)
	{
		var timeline_type_wrapper = Handlebars.compile($("#template-timeline_type_wrapper").html());

		var now = moment();
		var today = now.clone().hour(0).minute(0).second(0).millisecond(0);
		var int_treatment_index = 0;
		var obj_timeline_sections = [[], [], []];

		$.each(json_body.nutrition, function(i, item)
		{
			var item_timeslot = null;
			for (var i=0, len=pageTimeline.timeslots.length; i<len; i++)
			{
				if ( pageTimeline.timeslots[i].name.toLowerCase() == item.type.toLowerCase() )
				{
					item_timeslot = pageTimeline.timeslots[i];
					break;
				}
			}

			if ( item_timeslot )
			{
				var int_timeline_nutrition_section = 2;
				var date_item_minRange = moment(item_timeslot.minRange, "HH:mm");
				var date_item_maxRange = moment(item_timeslot.maxRange, "HH:mm");

				if ( now.isAfter(date_item_maxRange) && today.isSame(date) )
				{
					int_timeline_nutrition_section = 0;
				}
				else if ( ( now.isAfter(date_item_minRange) && now.isBefore(date_item_maxRange) && today.isSame(date) ) || ( now.isBefore(date_item_minRange) && obj_timeline_sections[1].length == 0 && today.isSame(date) ) )
				{
					int_timeline_nutrition_section = 1;
				}

				var b_nutrition_item_printed = false;
				b_medicines_items_printed = false;

				for (var j=int_treatment_index, len=json_body.medicines.length; j<len; j++)
				{
                    //debugger;
					var intake = json_body.medicines[j];

					// FIXME: El algoritmo de corte-control tenia un bug por el cual
					// si un timeslot no tenía un intake, en algunas circunstancias se perdían
					// algunos intakes de los timeslots posteriores.
					//
					// Por lo tanto, si el intake actual corresponde a un timeslot, que no es
					// el actual, NO HACEMOS NADA, para evitar procesar items que seguramente
					// pertenecen a otro timeslot
					//
					// Esto es sólo una medida temporal, lo ideal sería reescribir este algoritmo
					// por algo que no dependa de un corte-control y de datos ordenados
					if(intake.type !== "" && intake.type.toLowerCase() !== item.type.toLowerCase())
						break;

					var date_medicine = moment(intake.time, "HH:mm");
					if ( intake.type.toLowerCase() == item.type.toLowerCase() )
		            {
		            	var datos_wrapper = {
		            			type_class: "type_nutrition",
				            	collapsed: (int_timeline_nutrition_section == 2 && today.isSame(date)) ? "true" : "false",
				            	has_icon_class: Nire.connection.getEnabledSections().indexOf("TREATMENT") >= 0? "has_icon" : "",
				            	wrapper_title_icon: "<img src=\"img/others/ic_pill.png\" class='SECTION_TREATMENT' />",
				            	wrapper_title: gjson_lang["nutrition_meal_"+ item.type].toUpperCase()
		            	};

			            // Generamos los elementos del DOM
			            var type_wrapper = $(timeline_type_wrapper(datos_wrapper));

			            type_wrapper.find(".timeline_track_wrapper")
			            	.append(pageTimeline.getTrackItems(item.items, "nutrition", item.type))
			            	.append(pageTimeline.getTrackItems(intake.medicines, "treatment", item.type ));

			            obj_timeline_sections[int_timeline_nutrition_section].push( type_wrapper );
			            b_nutrition_item_printed = true;
					}
					else if ( !b_nutrition_item_printed && date_medicine.isAfter(date_item_maxRange) )
					{
						var datos_wrapper = {
								type_class: "type_nutrition",
								collapsed: (int_timeline_nutrition_section == 2 && today.isSame(date)) ? "true" : "false",
				            	has_icon_class: null,
				            	wrapper_title_icon: null,
				            	wrapper_title: gjson_lang["nutrition_meal_"+ item.type].toUpperCase()
		            	};
			            // Generamos los elementos del DOM
			            var type_wrapper = $(timeline_type_wrapper(datos_wrapper));

			            type_wrapper.find(".timeline_track_wrapper").append(pageTimeline.getTrackItems(item.items, "nutrition", item.type));

			            obj_timeline_sections[int_timeline_nutrition_section].push( type_wrapper );

			            b_nutrition_item_printed = true;

			            if (json_body.nutrition[i+1])
			            {
			            	int_treatment_index = j;
			            	continue;
			            }
					}
					else
					{
						var int_timeline_treatment_section = 2;
						if ( now.isAfter(date_medicine) && today.isSame(date) )
							int_timeline_treatment_section = 0;
						else if ( now.isBefore(date_medicine) && now.isBefore(date_item_minRange) && today.isSame(date) && obj_timeline_sections[1].length == 0 )
							int_timeline_treatment_section = 1;

						$.merge( obj_timeline_sections[int_timeline_treatment_section], pageTimeline.getTrackItems(intake.medicines, "treatment" ) );
						b_medicines_items_printed = true;
					}
					if ( b_nutrition_item_printed && i <= json_body.nutrition.length-1 )
					{
						int_treatment_index = j+1;
						break;
					}
					else if ( b_medicines_items_printed )
					{
						int_treatment_index = j+1;
					}
				}

				if ( !b_nutrition_item_printed )
				{
					var datos_wrapper = {
							type_class: "type_nutrition",
							collapsed: (int_timeline_nutrition_section == 2 && today.isSame(date)) ? "true" : "false",
			            	has_icon_class: null,
			            	wrapper_title_icon: null,
			            	wrapper_title: gjson_lang["nutrition_meal_"+ item.type].toUpperCase()
	            	};
		            // Generamos los elementos del DOM
		            var type_wrapper = $(timeline_type_wrapper(datos_wrapper));

		            type_wrapper.find(".timeline_track_wrapper").append(pageTimeline.getTrackItems(item.items, "nutrition", item.type));

		            obj_timeline_sections[int_timeline_nutrition_section].push( type_wrapper );
				}

			}
		});

		return obj_timeline_sections;
	},

	getTrackItems: function(ajson_tracks, track_type, item_type)
	{
		var timeline_track_item = Handlebars.compile($("#template-timeline_track_item").html());

		var date = $("#timeline_track_days_actual.dateselector").dateselector("date").format("YYYYMMDD");
		var date_previous = null;
        var a_tracks = [];

        $.each(ajson_tracks, function(i) {

        	var str_url = "#";
        	var str_url_params = null;
        	var str_img = null;
        	var str_title = null;
        	var str_icon_checked = null;
            var section_class = ""
            var extra_info = ""

        	this.track_type = track_type;
        	this.type = item_type? item_type.toLowerCase() : "";

        	if ( track_type == "nutrition" )
    		{
                section_class = "SECTION_NUTRITION"
                if(this.id.toLowerCase().indexOf("p") < 0){
        			str_url = "nutritionDetail.html";
        			str_url_params = "?content="+ encodeURIComponent(this.id) +"&date="+ encodeURIComponent(date) +"&type="+ encodeURIComponent(this.type);
    			}
                str_title = this.title;
    			str_icon_checked = "ui-icon-checked_nutrition";
                extra_info = this.quantity;
                if(this.percent) extra_info = extra_info + " " + this.percent;
    		}
        	else if ( track_type == "physicalActivity" )
    		{
                section_class = "SECTION_PHYSICAL"
        		if ( i == 5 )
        			return false;
                if(this.id.toLowerCase().indexOf("p") < 0){
        		  str_url = "physicalActivityDetail.html";
        		  str_url_params = "?content="+ encodeURIComponent(this.id) +"&date="+ encodeURIComponent(date) +"&type="+ encodeURIComponent(this.type);
        		}
                str_title = this.title;
                if(this.source) str_title += " (via " + this.source + ")"
    			str_icon_checked = "ui-icon-checked_physicalActivity";
    		}
    		else if (track_type == "treatment")
    		{
                section_class = "SECTION_TREATMENT"
    			str_url = "#";
    			str_img = "<img src=\"img/others/ic_pill.png\" class=\"ui-li-icon\" />";
    			str_title = this.title +" ("+ this.time +"h)";
    			str_icon_checked = "ui-icon-checked_treatment";
    			if ( date_previous != this.time && !item_type ){
    				date_previous = moment(this.time, "HH:mm");
    				var treatment_wrapper = $('<ul data-role="listview" data-split-icon="none" data-corners="false" data-shadow="false" class="timeline_track_wrapper has_box_shadow"></ul');
    			}

    		}
        	// Estos son los datos que requiere el template de Handlebars
            var datos_track_elem = {
            		track_type: "type_"+ track_type + " " + section_class,
            		url: str_url,
            		url_params: str_url_params,
            		img: str_img,
            		title: str_title,
	                icon_class: this.completed ? str_icon_checked : "ui-icon-unchecked",
                    extra_info: extra_info
            };
            // Generamos los elementos del DOM y le asociamos al información de este elemento
            // para despues poder recuperar toda la información del servidor
            var track_item = $(timeline_track_item(datos_track_elem)).jqmData("nire", this);

            if ( treatment_wrapper ) {
            	treatment_wrapper.html(track_item);
            	a_tracks.push( treatment_wrapper );
            } else {
            	a_tracks.push( track_item );
            }
        });

        return a_tracks;
	},

	prepareNutritionPreviousWrapper: function(duration)
	{
		if ( !pageTimeline.state.isPreviousWrapperShowed && duration )
		{
			pageTimeline.$previousStarter.stop().animate({ "margin-top": 0 }, { duration: duration, queue: false, done: function(){
				if ( pageTimeline )
				{
//					pageTimeline.$previousWrapper.css({ "position": "absolute", "visibility": "hidden" }).show();
					pageTimeline.$previousWrapper.css("visibility", "hidden").removeClass("in").show();
			    	$("#timeline_nutrition_previous_wrapper .ui-collapsible").collapsible("expand");
			    	$(this).css("visibility", "visible");
			    	$.mobile.resetActivePageHeight();
				}
			}}).show();
		}
		else if ( !pageTimeline.state.isPreviousWrapperShowed )
		{
//	    	pageTimeline.$previousWrapper.css({ "position": "absolute", "visibility": "hidden" }).show();
			pageTimeline.$previousWrapper.css("visibility", "hidden").removeClass("in").show();
	    	pageTimeline.$previousWrapper.find(".ui-collapsible").collapsible("expand");
	    	pageTimeline.$previousStarter.css({"visibility": "visible", "margin-top": 0 }).show();
	    	$.mobile.resetActivePageHeight();
		}
	},

	showNutritionPreviousWrapper: function()
	{
		var int_duration = (pageTimeline.state.isPreviousWrapperShowed) ? 0 : 700;
		pageTimeline.$previousStarter.css("visibility", "hidden");
		pageTimeline.$previousWrapper.css("visibility", "visible");
		pageTimeline.$previousStarter.stop().animate(
				{"margin-top": pageTimeline.$previousWrapper.outerHeight(true) - pageTimeline.$previousStarter.height() },
				{"duration": int_duration, "queue": false, done: function(){
					pageTimeline.$previousStarter.css("margin-top", - pageTimeline.$previousStarter.height());
//					pageTimeline.$previousWrapper.css({ "position": "static" });
					pageTimeline.$previousWrapper.addClass("in");
					$.mobile.resetActivePageHeight();
					pageTimeline.state.isPreviousWrapperShowed = true;
				}}
			);
	},

    resetTracksPosition: function(time, successCallback)
	{
		var int_time = (time == 0) ? 0 : time || pageTimeline.animationDuration;
		$("#timeline_animated_content").stop().animate( { "left": 0, "right": 0 }, { duration: int_time, queue: false } )
			.promise()
			.done(successCallback);
	},

    prepareTracksForAnimation: function()
	{
		var int_left = parseInt($("#timeline_animated_content").css("left"));
		if (Math.abs(int_left) != 0) {
			$("#timeline_animated_content").hide().trigger("updatelayout");;
			$("#timeline_animated_content").css( { "left": 0, "right": 0 } );
			$("#timeline_track_days_actual.dateselector .day_link").removeClass("ui-disabled");
			return true;
		}
		$("#timeline_track_days_actual.dateselector .day_link").removeClass("ui-disabled");
		return false;
	},

	animateTracks: function(direction, diff_days)
	{
		$("#timeline_track_days_actual.dateselector .day_link").addClass("ui-disabled");
		var int_window_width = null;
		var str_what_date = null;

		if (direction == "right") {
			int_window_width = $( window ).width();
			str_what_date = "previous";
		}
		else if ( direction == "left") {
			int_window_width = -($( window ).width());
			str_what_date = "next";
		}
		var obj_what_date = (diff_days) ? $("#timeline_track_days_actual.dateselector").dateselector("date").clone().add(diff_days, "day") : null;

		if (int_window_width) {
			$("#timeline_animated_content")
				.stop()
				.animate(
					{ "left": int_window_width, "right": -(int_window_width) },
					{ duration: pageTimeline.animationDuration, queue: false }
				)
				.promise()
				.done(function(){
					if (obj_what_date) {
						if ( !$("#timeline_track_days_actual.dateselector").dateselector("date", obj_what_date)) {
							pageTimeline.resetTracksPosition();
						}
					} else {
						if ( !$("#timeline_track_days_actual.dateselector").dateselector(str_what_date)) {
							pageTimeline.resetTracksPosition();
						}
					}
				})
				.fail(function(){
					pageTimeline.resetTracksPosition();
				});
		}
	},

	/**
     * Marcar un Track como realizado.
     * Por defecto hace "switch" (marca o desmarca alternativamente) pero si se
     * le pasa 'false' en switchStatus, sólo lo marcará.
     * Necesitamos esto para conectar cosas como el "I took it" de los mensajes de
     * notificación de tomas de medicamento.
     */
    updateTrack: function($element, notificationId, switchStatus)
    {
        var self = this;
        var data = $element.jqmData("nire");
        
        if ( switchStatus === false && data.completed == true ) return false;
        var date = $("#timeline_track_days_actual.dateselector").dateselector("date");
        if( data.section == "treatment" )
        {
            date = data.fullTime;
        }

        Nire.analytics.event( "click", "Timeline track update" );
        vibrate();
        $element.children().last().addClass('rotatingcheck');
        var t0 = Date.now();

        var trackTypeToSend = data.section || data.track_type

        if(data.isUserAdded) {
            Nire.connection.deleteNutritionTrack(date, data.type, data.id)
                .done( function() { self.deleteTrackVisual($element) } )
        }
        else {
            Nire.connection.updateTrack(date, data.id, trackTypeToSend, data.type)
                .done(function(result){
                    pageTimeline.markTrackCompleted(result, $element, notificationId);
                    setTimeout(function() {
                        $element.children().last().removeClass('rotatingcheck');
                    }, (1000 - (Date.now()-t0)));
                })
                .fail(function(result){
                    setTimeout(function() {
                        $element.children().last().removeClass('rotatingcheck');
                    }, 2000);
                });
        }
    },

    /**
     * Quita de la pantalla un track.
     */
    deleteTrackVisual: function($element) {
        $element.slideUp();
    },


    markTrackCompleted: function(result, $element, notificationId)
    {
        var classAdd, classRmv;

        var data = $element.jqmData("nire");
        if(result && result.data && result.data.status == "confirmed"){
            classAdd = 'ui-icon-checked_'+ data.track_type;
            classRmv = 'ui-icon-unchecked';
        } else {
            classRmv = 'ui-icon-checked_'+ data.track_type;
            classAdd = 'ui-icon-unchecked';
            if (result && result.error && result.error.message == "DATE_AFTER_TODAY") {
            	showToast(gjson_lang["error_"+ data.track_type +"_updatetrack_date_after_today"]);
            }
        }

        $element.find(".update_track").removeClass(classRmv).addClass(classAdd);
        
        // Si no es de notificación no encontrará el elemento y no pasa nada
        $("#page_timeline #"+ notificationId).remove();
    },
    
    getHealthIndicators: function()
    {
    	showLoading();

    	Nire.connection.health()
    		.done(function(json_response)
    		{
				if (json_response.error.code == 0)
                {
					pageTimeline.addHealthIndicators(json_response.data);
                }
				else
				{
					showAlert(json_response.error.message, null, gjson_lang.error, gjson_lang.accept);
                }
    		})
    		.always(function()
    		{
    			hideLoading();
    		});
    },
    
    addHealthIndicators: function(json)
    {
    	if ( json.body.indicators.length > 0 )
    	{
    		pageTimeline.createHealthIndicators(json.body.indicators);
    		$("#timeline_measures_last_action").text(gjson_lang["timeline_last_measures_action_hide"]);
    		$("#timeline_measures_last_action").jqmData("action-hide", true);
    	}
    },
    
    createHealthIndicators: function(ajson_measures)
    {
    	var template_measures_last = Handlebars.compile($("#template-timeline_measures_last").html());
    	
    	// Comprobamos si es un indicador que ya está añadido
    	// TODO: add nire jqmData and use filterByNireData -> change handlebars template
    	var a_measures_last_displayed = [];
    	$("#timeline_measures_last_wrapper .ui-grid-b:nth-child(2) .timeline_health_indicator").each(function(i)
    	{
    		a_measures_last_displayed.push(getDataURLParameter($(this), "content") || getDataURLParameter($(this), "value"));
    	});
    	
    	var a_block_class = ["a","b","c"];

        var a_measures = [];
        var int_real_cont = 0;
        
        $.each(ajson_measures, function(i, item)
        {
        	if ( a_measures_last_displayed.indexOf(item.type) >=0 ) return true;
        	
        	var str_url = null;
        	var str_url_parmas = null;
        	var str_disabled_class = null;
        	var str_quantity = null;
        	var str_unit = null;
        	
        	if ( this.active )
        	{
        		str_url = "indicatorDetail.html";
        		str_url_parmas = "?content="+ this.type;
        		str_quantity = this.value;
        		str_unit = this.unit;
        	}
        	else
        	{
        		str_url = "newTrackHealth.html";
            	str_url_parmas = "?value="+ this.type;
            	str_quantity = '<img src="img/others/ic_nutrition_plus.png" width="40" height="40" />';
            	str_disabled_class = "ui-btn-active";
        	}
        	// Agregamos la medida a la lista de medidas que se mostrarán
            a_measures.push({
            	block_class: "ui-block-"+ a_block_class[int_real_cont % 3],
            	url: str_url,
            	url_params: str_url_parmas,
            	disabled_class: str_disabled_class,
            	value_class: ( this.alert ) ? "color_txt_danger" : "color_txt_nutrition",
            	quantity: str_quantity,
            	unit: str_unit,
                name: i18n(this.type),
            });
            
        	if ( int_real_cont % 3 == 2 || ajson_measures.length-1 == i)
            {
            	// Estos son los datos que requiere el template de Handlebars
                var datos_wrapper =  {
                    grid_class: "ui-grid-b",
                    last_measures: a_measures
                };                
                // Añadimos los nuevos indicadores pero al aprovechar la plantilla template-timeline_measures_last 
            	// hay que eliminar una parte de la misma que no nos sirve
                $("#timeline_measures_last_wrapper").append(template_measures_last(datos_wrapper)).trigger( "updatelayout" );
                // Vacíamos el array de indicadores para comenzar el nuevo grid
                a_measures = [];
            }
        	int_real_cont++;
        });
    },

    getNotificationActionPath: function(action, value, id)
	{
		var str_path = null;

		switch(action.toLowerCase()) {
		    case "nutrition":
		        str_path = {
		    		url: "newTrackNutrition2.html"
		    	};
		    break;
		    case "physicalactivity":
		    	str_path = {
		    		url: "newTrackPhysicalActivity.html"
		    	};
		    break;
		    case "health":
		    	str_path = {
		    		url: "newTrackHealth.html"
		    	};
		    break;
		    case "measure":
		    	str_path = {
		    		url: "newTrackHealth.html"
		    	};
		    break;
		    case "question":
		    	str_path = {
		    		url: "#",
		    		params: "?action="+action.toLowerCase() + "&value="+ value +"&notificationId=" + id
		    	};
		    break;
		    case "confirm":
		    	var a_value = value.split("#");
		    	str_path = {
		    		url: "#",
		    		params: "?action="+action.toLowerCase() +"&id="+ a_value[1] +"&date="+ a_value[2]  +"&notificationId=" + id
		    	};
		    break;
		    default:
		    	str_path = {
		    		url: action
		    	};
		}

		return str_path;
	},

	getNewQuestion: function(id, notificationId)
	{
		showLoading();
        Nire.connection.getPreguntaNueva(id)
        	.done(function(data) {
                pageTimeline.question = data;
                pageTimeline.notificationId = notificationId;
        		pageTimeline.createQuestionPopup(data);
            })
            .always(function(){
            	hideLoading();
            });
	},

	createQuestionPopup: function(json_question)
	{
		var response_options = Handlebars.compile($("#template-timeline_adds_on_type_notification_question_popup_" + json_question.type).html());
		var popup = Handlebars.compile($("#template-timeline_adds_on_type_notification_question_popup").html());

		var datos_popup = {
				title: json_question.label,
				response_html: response_options(json_question),
				send_btn_text: gjson_lang.medwhat_consultor_text
			};

		pageTimeline.$page.find(".ui-content").append(popup(datos_popup));

		$("#timeline_add_on_type_notification_question_popup").find("div:jqmData(role=footer)").toolbar();
		$("#timeline_add_on_type_notification_question_popup").find("fieldset:jqmData(role=controlgroup)").controlgroup();
		$("#timeline_add_on_type_notification_question_popup").find("input[type=text]").textinput();
		$("#timeline_add_on_type_notification_question_popup").popup().popup("open");
	},

    questionResponse: function()
    {
        var respuesta = $("#timeline_add_on_type_notification_question_popup_response").serialize();

        var valor = getQuestionResponseValue(respuesta, this.question)
        if(valor == null) return;

        if( validarRespuesta(this.question.validator, valor)) {
            Nire.connection.respuesta(this.question.id, valor, this.notificationId)
            	.done(function(){
            		$("#page_timeline  #"+ pageTimeline.notificationId).remove();
            	})
            	.always(function(){
            		$("#timeline_add_on_type_notification_question_popup").popup("close");
            		Nire.connection.regenerateDietPetiton();
            	});

            Nire.analytics.event( "click", "TimelineNotificationQuestion answered", "Question: " + this.question.id );
        }
    },

    checkHelp : function(data)
    {
    	if (data)
    	{
            // Si todos los id de las imagenes ya fueron vistos, no mostramos la ayuda.
            // Si falta ver alguno, mostramos la ayuda
            var viewed = this.getViewedImages();
            var current = data.map(function(item) { return item.id })

            var allViewed = current.every(function(id) { return viewed.indexOf(id) !== -1})
            if(!allViewed) {
            	pageTimeline.cancelGetData();
                $( "body" ).pagecontainer( "change", "appHelp.html?onlynew=true" );
            }
    	}

    },

    getViewedImages: function()
    {
        var strJson = Nire.localStorage.getItem("viewed_app_help");
        var result = strJson ? JSON.parse(strJson) : [];

        return result;
    },

    addCover: function()
    {
        //Attaching the code of the cover to de code of the home
        $('#cover').load("cover_home.html", function(){
            $("body").css({ overflow: 'hidden' });
            $("body").addClass('ui-disabled');
            $("#cover_home").css("height", $(window).height() );
            $('#cover p.cover_wait_generating').text(gjson_lang["cover_wait_generating"]);
            $('#cover p.cover_wait_last').text(gjson_lang["cover_wait_last"]);
        });
    },

    emptyCover: function()
	{
        $('#cover').empty();
        $("body").css( "overflow", 'initial');
        $("body").removeClass('ui-disabled');
	},

	formatMedicineTime: function(date) {
		var today = moment().hour(0).minute(0).second(0).millisecond(0);
		var take = date.clone().hour(0).minute(0).second(0).millisecond(0);
        var formatted = "";
		if (!take.isSame(today))
			formatted = formatHeaderDate(take, today) + " ";

		return formatted + date.format("HH:mm");
	}
};

/* EVENTS */

$( window ).off( ".timeline" )
	.on("resize.timeline")
	{
		if ( pageTimeline && pageTimeline.$page)
		{
			pageTimeline.$page.swipe("option", "threshold", $(window).width()/4);
		}
	}

$( document ).off( ".timeline" )

	.on( "pagecreate.timeline", "#page_timeline", function()
    {
		pageTimeline.$page = $(this);
		pageTimeline.$previousWrapper = $("#timeline_nutrition_previous_wrapper");
		pageTimeline.$previousStarter = $("#timeline_nutrition_previous_starter");

        var date = (moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD").isValid()) ? moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD") : moment().hour(0).minute(0).second(0);
        $( "#timeline_img_background" ).css("background-image", "url(img/backgrounds/" + date.isoWeekday() + ".jpg)").trigger('updatelayout');		
		
		var str_start_direction = null;

		$( this ).swipe({
			swipeDown: function()
			{
				if (pageTimeline.previousWrapperStarter)
				{
					pageTimeline.previousWrapperStarter = false;
					$( document ).off("touchmove.timeline");
					pageTimeline.showNutritionPreviousWrapper();
				}
			},
			swipeLeft: function()
			{
				pageTimeline.animateTracks("left", 1);
				pageTimeline.state.isPreviousWrapperShowed = false;
			},
			swipeRight: function()
			{
				pageTimeline.animateTracks("right", -1);
				pageTimeline.state.isPreviousWrapperShowed = false;
			},
			swipeStatus: function( event, phase, direction, distance, duration, fingers )
			{
				if ( phase == "move" )
				{
					if ( !str_start_direction )
					{
						str_start_direction = direction;
						if ( str_start_direction == "left" || str_start_direction == "right" )
						{
							console.log("str_start_direction: left or right");
							pageTimeline.$page.swipe("option", "allowPageScroll", "none");
						}
						else if (!pageTimeline.previousWrapperStarter && str_start_direction == "down" && $(window).scrollTop() <= 20 
								&& pageTimeline.$previousStarter.css("visibility") == "visible" && pageTimeline.$previousWrapper.children().length > 0 )
						{
							console.log("str_start_direction: down or direction: up");
							pageTimeline.$previousStarter.css("visibility", "hidden");
							pageTimeline.$previousWrapper.css("visibility", "visible");
							pageTimeline.previousWrapperStarter = true;
							pageTimeline.$page.swipe("option", "allowPageScroll", "none");
							$( document ).on("touchmove.timeline", function(event){event.preventDefault();});
						}
					}
					
					if ( pageTimeline.previousWrapperStarter && str_start_direction == "down" && (direction == "down" || direction == "up") )
					{
						console.log("start down, move down/up");
						pageTimeline.$previousStarter.css( { "margin-top": distance +"px"} );
					}
					else if ( direction == "left" && (str_start_direction == "left" || str_start_direction == "right") )
					{
						console.log("left");
						$("#timeline_animated_content").css( { "left": -(distance), "right": distance} );
					}
					else if ( direction == "right" && (str_start_direction == "left" || str_start_direction == "right") )
					{
						console.log("right");
						$("#timeline_animated_content").css( { "left": distance, "right": -distance } );
					}
					else
					{
						console.log("move else");
						return false;
					}
				}
				else if ( phase == "cancel" )
				{
					console.log("cancel");
					pageTimeline.resetTracksPosition();
					if ( pageTimeline.previousWrapperStarter )
						pageTimeline.prepareNutritionPreviousWrapper(500);
					pageTimeline.$page.swipe("option", "allowPageScroll", "vertical");
					$( document ).off("touchmove.timeline");
					str_start_direction = null;
					pageTimeline.previousWrapperStarter = false;
				}
				else if ( phase == "end" )
				{
					console.log("end");
					pageTimeline.$page.swipe("option", "allowPageScroll", "vertical");
					if (direction != "left" && direction != "right" && !pageTimeline.previousWrapperStarter)
					{
						pageTimeline.resetTracksPosition();
					}
					str_start_direction = null;
				}
			},
			allowPageScroll: "vertical",
			cancelThreshold: $( window ).width()/8,
			threshold: $( window ).width()/4,
			excludedElements: ".noSwipe"
		});
    })

    .on("pagecontainershow.timeline", function(event, ui)
    {
    	if (ui.toPage.attr("id") == "page_timeline")
    	{
    		pageTimeline.createHeader();
    	}
    })

    .on( "pagecontainerchange.timeline", function(event, ui)
	{
    	if (ui.toPage.attr("id") == "page_timeline")
    	{
    		menuHome.getData().done(function()
    		{
	    		$("#timeline_track_days_actual.dateselector").dateselector({
	    			date: (moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD").isValid()) ? moment(Nire.localStorage.getItem(APP_DATE_STATE), "YYYYMMDD") : null,
	    			from_date: null,
		            minDate: Nire.rangeData.minDate.clone().hour(0).minute(0).second(0).millisecond(0),
		            maxDate: Nire.rangeData.maxDate.clone().hour(0).minute(0).second(0).millisecond(0),
		            dateFormatter: function(){
		            	return null;
		            },
		            change: function(event, newdate) {
		            	Nire.analytics.event( "datechange", "Timeline date switch" );
		            	Nire.localStorage.setItem(APP_DATE_STATE, newdate.format("YYYYMMDD"));
		            	pageTimeline.getData(newdate);
		            },
		            fn_date_clicked: function(obj_new_date) {
		            	pageTimeline.state.isPreviousWrapperShowed = false;
			    		var obj_from_date = $("#timeline_track_days_actual.dateselector").dateselector( "date" );
			    		var int_diff_days = obj_new_date.diff(obj_from_date, 'days');
			    		if ( obj_new_date.isAfter(obj_from_date) ){
			    			pageTimeline.animateTracks("left", int_diff_days);
			        	} else if ( obj_new_date.isBefore(obj_from_date) ){
			    			pageTimeline.animateTracks("right", int_diff_days);
			        	}
			    		return false;
			    	}
		        });
    		});
    	}
    	else if ( ui.prevPage.attr("id") == "page_timeline" && ui.toPage.attr("id") != "page_timeline"
					&& ui.toPage.attr("id") != "page_app_direct_communicator" )
    	{
			$("#page_timeline").trigger( "pageremove" );
			$("#page_timeline").remove();
		}
	})

    .on("pageremove.timeline", "#page_timeline", function()
	{
    	$("body").css("overflow", "auto");
    	pageTimeline.cancelGetData();
    	$( window ).off( ".timeline" );
    	$( document ).off( ".timeline" );
		$( 'link[href="css/pageTimeline.css"]' ).remove();
		$( 'link[href="css/pageCoverHome.css"]' ).remove();
		pageTimeline = null;
	})

	.on("scrollstop.timeline", function(event)
	{
		var window_scrolltop = $( window ).scrollTop();
		var wrapper_height = pageTimeline.$previousWrapper.outerHeight(true) - pageTimeline.$previousStarter.height();
		if ( pageTimeline.$previousWrapper.css("visibility") == "visible" && window_scrolltop >= wrapper_height )
		{
			pageTimeline.state.isPreviousWrapperShowed = false;
			pageTimeline.prepareNutritionPreviousWrapper();
			$(window).scrollTop(window_scrolltop - wrapper_height);
			return false;
		}
	})

	.on("click.timeline", "#timeline_content_actions_display_wrapper", function(event)
	{
		event.preventDefault();
		pageTimeline.$page.find(".plus-ex").click();
		return false;
	})

	.on("click.timeline", "#page_timeline .plus-ex", function(event)
	{
		event.preventDefault();
		if( $(this).hasClass('plus') )
		{
			$('<div id="timeline_content_actions_display_wrapper" class="ui-popup-screen ui-overlay-b in"></div>').appendTo("#page_timeline");
			$("body").css("overflow", "hidden");
			pageTimeline.$page.swipe("disable");
			$(this).removeClass('plus');
			$(this).addClass('ex');
			$('#moment-buttons li').addClass('out');
		}
		else
		{
			$('#moment-buttons li').removeClass('out');
			$("body").css("overflow", "auto");
			pageTimeline.$page.swipe("enable");
			$(this).removeClass('ex');
			$(this).addClass('plus');
			$("#timeline_content_actions_display_wrapper").remove();
		}
		return false;
	})
	
	.on("click.timeline", "#timeline_measures_last_action", function(event)
	{
		event.preventDefault();
		
		var int_ui_grid_solo_count = $("#timeline_measures_last_wrapper .ui-grid-solo").length;
		
		if ( !$(this).jqmData("action-hide") && int_ui_grid_solo_count == 1 )
		{
			pageTimeline.getHealthIndicators();
		}
		else if ( !$(this).jqmData("action-hide") && int_ui_grid_solo_count > 1 )
		{
			$("#timeline_measures_last_wrapper div[class^=ui-grid-]").slice(2).show().trigger("updatelayout");
			$(this).text(gjson_lang["timeline_last_measures_action_hide"]);
			$(this).jqmData("action-hide", true);
		}
		else
		{
			$("#timeline_measures_last_wrapper div[class^=ui-grid-]").slice(2).hide().trigger("updatelayout");
			$(this).text(gjson_lang["timeline_last_measures_action"]);
			$(this).jqmData("action-hide", null);
		}
		
		return false;
	})

    .on("click.timeline", "#page_timeline .update_track", function(event)
    {
    	event.preventDefault();

        pageTimeline.updateTrack($(this).parent(".timeline_track_item"));

        return false;
    })

    .on("click.timeline", ".timeline_add_on_notification_dismiss", function(event)
	{
    	event.preventDefault();
    	$(this).closest(".timeline_add_on_notification").fadeOut(700, function(){
    		$(this).remove();
    	});
        
        var notificationsViewed = Nire.localStorage.getJSONItem("notificationsViewed");
        if(!notificationsViewed) notificationsViewed = [];

        notificationsViewed[notificationsViewed.length] = Number($(this).closest(".timeline_add_on_notification").attr("id"));
        Nire.localStorage.setJSONItem("notificationsViewed", notificationsViewed);

    	return false;
    })

    .on("click.timeline", "#page_timeline .timeline_add_on_notification_action", function(event)
	{
    	event.preventDefault();
        var actionElement = $(this);
        $(this).closest(".timeline_add_on_notification").fadeOut(700, function(){
            var action = getDataURLParameter($(actionElement), "action");
            if ( $(actionElement).attr("href") == "#" && action === "question" ) {
                showLoading();
                pageTimeline.getNewQuestion(getDataURLParameter($(actionElement), "value"), getDataURLParameter($(actionElement), "notificationId"));
            }
            else if ( $(actionElement).attr("href") == "#" && action === "confirm" )
            {
                var $element = $(".timeline_track_item.type_treatment").filterByNireData("id", getDataURLParameter($(actionElement), "id"));
                pageTimeline.updateTrack($element, getDataURLParameter($(actionElement), "notificationId"), false);
            }
            $(this).remove();
        });

		return false;
    })

    .on("click.timeline", "#timeline_add_on_type_notification_question_popup_send_btn", function(event)
	{
    	event.preventDefault();
    	pageTimeline.questionResponse();
    	return false;
    })

	.on("click.timeline", ".timeline_track_item.type_treatment > .timeline_track_title", function(event)
	{
		event.preventDefault();

		var template_treatment_popup = Handlebars.compile($("#template-timeline_treatment_popup").html());

		var data = $(this).parent().jqmData("nire");

		var datos_popup = {
				treatment: data.treatment,
				title: data.title,
				presentation: data.presentation,
				dosage: data.dosage,
				description: data.description,
				action_text: gjson_lang["pillbox_footer"]
			};

		pageTimeline.$page.find(".ui-content").append(template_treatment_popup(datos_popup));

		$('#timeline_treatment_popup div:jqmData(role=header)').toolbar();
    	$('#timeline_treatment_popup').popup().popup("open");
    	return false;
	})

    .on("click.timeline", "#btn_pending_notifications:visible", function(event)
	{
    	event.preventDefault();
 		cloneAppHeader(pageTimeline.$page);
		$( "body" ).pagecontainer("change", "appDirectCommunicator.html", { transition:"slide" });
		return false;
	})

	.on( "popupbeforeposition.timeline", ".ui-popup", function()
    {
    	$("body").css("overflow", "hidden");
    	var maxWidth = $( window ).width() - 50 + "px";
    	var maxHeight = $( window ).height() - 100 + "px";
		$( this ).css({ "max-height": maxHeight, "max-width": maxWidth });
		pageTimeline.$page.swipe("disable");
    })

    .on( "popupafterclose.timeline", ".ui-popup", function()
    {
    	$("body").css("overflow", "auto");
        $( this ).remove();
		pageTimeline.$page.swipe("enable");
    })
    ;