/**
 * Ejecutar varias peticiones "asincronas" simultaneamente, pero limitada a una determinada cantidad.
 */
Nire.AsyncQueue = function(threads) {
	this.queue = [];
	this.running = 0; // Cantidad de procesos ejecutandose actualmente.

	if(!threads) threads = ASYNC_QUEUE_SIZE | 3;
	this.threads = threads;
}

/**
 * Agregar una funcion a la lista de funciones a ejecutar. Todas las funciones que se agregan tienen
 * como requerimiento retornar un Promise, el cual será utilizado para detectar cuando termina de 
 * ejecutarse.
 *
 * @param _function función que se ejecuta asincrónicamente y retorna un promise. Si se quieren pasar
 *                  parametros a la función, o controlar el valor de this, se aconseja utilizar function::bind
 */
Nire.AsyncQueue.prototype.add = function( _function ) {
	this.queue.push( _function );
	this._startQueue();
}

Nire.AsyncQueue.prototype._startQueue = function() {
	// si no se llegó al limite de concurrencia, lanzo un nuevo proceso concurrente.
	if(this.running < this.threads) {
		this.running++;
		this._startProcess();
	}
}

Nire.AsyncQueue.prototype._startProcess = function() {
	// Quita un elemento de la cola y lo procesa, cuando termina vuelve a iniciar el proceso.
	// No debemos preocuparnos por problemas de asincronismo porque javascript es sincronico siempre.
	var _this = this;

	var _function = this.queue.pop();
	if(_function !== undefined) {
		_function().always(function() {
			_this._startProcess();
		});
	}
	else {
		// No queda nada en la cola, entonces descontamos este consumer
		_this.running--;
	}
}