/**
 * Logica para recupera, mostrar y almacenar notificaciones
 */
var Nire = Nire || {}
Nire.Notification = function(connection) {
	this.connection = connection;	
	this.inServerCall = false;
	this.log = getLogger("Notification");
}

/**
 * Recuperar el identificador de la ultima notificación leída
 */
Nire.Notification.prototype.getIdLastNotification = function() {
	if(typeof ENTORNO !== "undefined" && ENTORNO == "DESARROLLO") return 0;
	
	var lastNotification = Nire.localStorage.getItem( this._idLastNotificationKeyName() );
	if(lastNotification == null) lastNotification = 0;

	return lastNotification;
}

/**
 * Guardar el identificador de la ultima notificacion leída
 */
Nire.Notification.prototype.setIdLastNotification = function(id) {
	Nire.localStorage.setItem( this._idLastNotificationKeyName(), id );
}

/**
 * Recuperar la cantidad de notificaciones pendientes
 */
Nire.Notification.prototype.getPendingCount = function(count) {
	return Nire.localStorage.getItem( this._pendingCountKeyName() ) || 0;
}

/**
 * Guardar la cantidad de notificaciones pendientes
 */
Nire.Notification.prototype.setPendingCount = function(count) {
	Nire.localStorage.setItem( this._pendingCountKeyName(), count);
}

Nire.Notification.prototype.getLastOrUnanswered = function() {
	var previous = JSON.parse(Nire.localStorage.getItem( this._getStorageKeyname() ));
	if(!previous) return null;

	var responses = JSON.parse(Nire.localStorage.getItem( this._getResponseKeyname() )) || {};
	var notificationsViewed = Nire.localStorage.getJSONItem("notificationsViewed") || [];
	var lastViewed = this.getIdLastNotification()

	var pendingNotifications = [];

	for(var idx in previous) {
		var id = previous[idx].id;
		previous[idx].title = gjson_lang.timeline_message_title
		if(notificationsViewed.indexOf(id) == -1 && id > Number(lastViewed)) {
			pendingNotifications.push({type: "notification", content: previous[idx]})
		}else if(!responses[String(id)] && (previous[idx].button || previous[idx].action)){
			pendingNotifications.push({type: "notification", content: previous[idx]})
		}
	}

	if(pendingNotifications.length > 0)
		pendingNotifications[pendingNotifications.length - 1].priority = 1
	
	return pendingNotifications.slice(pendingNotifications.length - 2);
}

/**
 * Recuperar todas las notificaciones.
 */
Nire.Notification.prototype.get = function() {
	var previous = JSON.parse(Nire.localStorage.getItem( this._getStorageKeyname() )) || [];
	var responses = JSON.parse(Nire.localStorage.getItem( this._getResponseKeyname() )) || {};
	var deferred = $.Deferred();

	deferred.resolve()

	return deferred.then(function() { return { data: previous, responses: responses}});
}

Nire.Notification.prototype.getNewNotifications = function() {
	var previous = JSON.parse(Nire.localStorage.getItem( this._getStorageKeyname() )) || [];
	var responses = JSON.parse(Nire.localStorage.getItem( this._getResponseKeyname() )) || {};

	var recent = []
	for(var idx in previous){
		if(previous[idx].id > this.getIdLastNotification())
			recent.push(previous[idx])
	}
	if(recent.length > 0)
		this.setIdLastNotification( recent[recent.length-1].id );

	//como ya obtuve todas las notificaciones pendientes, 
	//elimino la lista de notificaciones ignoradas del timeline
	Nire.localStorage.removeItem("notificationsViewed");

	var deferred = $.Deferred();
	deferred.resolve( { data: recent, responses: responses } )
	return deferred.promise();
}

Nire.Notification.prototype.getRecent = function() {
	return this._retrieve().then(function(data) { return { data: data.recent, responses: data.responses}; });
}

/**
 * Borra todos los mensajes similares al mensaje dado.
 * El mensaje que se identifica por el parámetro no es borrado, todo los demás si
 */
Nire.Notification.prototype.deleteRepeatedQuestions = function(idNotification) {
    var questions = JSON.parse(Nire.localStorage.getItem( this._getStorageKeyname() )) || [];
    var responses = JSON.parse(Nire.localStorage.getItem( this._getResponseKeyname() )) || {};

    var questionMessage = questions.filter(function(obj) { return obj.id == idNotification })[0].message;
    var questionsToDelete = questions.filter(function(obj) { return obj.id != idNotification && obj.message == questionMessage });

    questionsToDelete.forEach(function(obj) { delete responses[obj.id]; });
    questions = questions.filter(function(obj) { return questionsToDelete.indexOf(obj) == -1; })

	Nire.localStorage.setItem( this._getStorageKeyname(), JSON.stringify(questions));
	Nire.localStorage.setItem( this._getResponseKeyname(), JSON.stringify(responses));
}

/**
 * Insertar un mensaje a la lista de mensajes.
 */
Nire.Notification.prototype._insertMessage = function( message ) {
	var messages = JSON.parse(Nire.localStorage.getItem( this._getStorageKeyname() )) || [];
	messages.push(message);
	Nire.localStorage.setItem( this._getStorageKeyname(), JSON.stringify(messages));

	return messages;
}

Nire.Notification.prototype._retrieve = function( ) {
	var _this = this;

	// Do not allow two simultaneus server calls
	if(this.inServerCall) {
		var deferred = $.Deferred();
		deferred.reject();
		return deferred.promise();
	}
	this.inServerCall = true;	

	return this.connection.retrieveNotifications( this.getIdLastNotification() ).then(function(json) {
		var previous = JSON.parse(Nire.localStorage.getItem( _this._getStorageKeyname() )) || [];
		previous.push.apply(previous, json.notifications);

		var responses = JSON.parse(Nire.localStorage.getItem( _this._getResponseKeyname() )) || {};
		var lastResponse = 0
		for (var attrname in json.responses) { 
			responses[attrname] = json.responses[attrname];
			lastResponse = Math.max(json.responses[attrname].id, lastResponse)
		}

		Nire.localStorage.setItem( _this._getStorageKeyname(), JSON.stringify(previous));
		Nire.localStorage.setItem( _this._getResponseKeyname(), JSON.stringify(responses));

		_this.setPendingCount(0);		

		_this.inServerCall = false;
		
		return { history: previous, recent: json.notifications, responses: responses };
	}).fail(function(){_this.inServerCall = false;});
}

Nire.Notification.prototype._getStorageKeyname = function() { 
	return this._prefixKeyName("notifications:storage");
}

Nire.Notification.prototype._getResponseKeyname = function() { 
	return this._prefixKeyName("responses:storage");
}

Nire.Notification.prototype.addAnswer = function(notificationId, reply) {
	var responses = JSON.parse(Nire.localStorage.getItem( this._getResponseKeyname() )) || {};
    responses[notificationId] = {
        message: reply,
        notification: notificationId
    }
    Nire.localStorage.setItem( this._getResponseKeyname(), JSON.stringify(responses));

}

Nire.Notification.prototype._prefixKeyName = function(suffix) {
	return suffix;
}

Nire.Notification.prototype._idLastNotificationKeyName = function() {
	return this._prefixKeyName("notifications:last");
}

Nire.Notification.prototype._pendingCountKeyName = function() {
	return this._prefixKeyName("notifications:pending");
}

