/**
 * Monitorear el servidor por medio de push u otro método establecido invalidaciones y/o comandos
 * que envie el servidor. Por ejemplo invalidacion de caché.
 *
 * @param connection nexo de comunicacion con el servidor. Debe tener un metodo getServerStatus e invalidateCache
 */
Nire.ServerMonitor = function(connection) {
	if(!connection) throw "Parameter connection can not be nothing."

	this.connection = connection;
	this.listeners = [];
	this.log = getLogger("ServerMonitor");
}

/**
 * Agrega un nuevo listener para realizar acciones cuando se llama a ServerStatus
 */
Nire.ServerMonitor.prototype.addListener = function( listener ) {
	// TODO: Verificar que el mismo listener ya no este agregado a la lista.
	this.listeners.push(listener);
}

/**
 * Detener la comprobación hasta que se reanue nuevamente por medio del método start
 */
Nire.ServerMonitor.prototype.stop = function() {
	if(this.timerId) clearInterval(this.timerId);
}

/**
 * Inciar la comprobación cada cierto intervalo. Si no se especifica el intervalo
 * se utilizara el ulitimo empleado
 */
Nire.ServerMonitor.prototype.start = function() {
	if(this.timerId) this.stop();
	this.log("Iniciando monitoreo del servidor con un intervalo fibonacci");
	
	var fibonacciPosition = 1;
	var self = this;

	var callback = function() {
		self._timerCallback();

		if(fibonacciPosition < 13)
			fibonacciPosition = fibonacciPosition + 1;

		var next = fibonacci(fibonacciPosition);
		self.timerId = setTimeout(callback, next*1000);
		self.log("Proximo callback en (segundos)", next);
	}

	callback();
}

Nire.ServerMonitor.prototype._timerCallback = function() {
	// Lo único que nos importa es llamar algun action del servidor que retorne los headers
	// necesarios. Esta llamada será encausada por _serverCall el cual detectará que estan
	// los headers necesarios e invocará notifyObservers.
	//
	// Antes se llamaba a notifyObservers aquí, pero esto causaba que se llamara dos veces
	// o que getServerStatus tenga que hacer una excepción y no ir por _serverCall
	this.connection.getServerStatus()
}

Nire.ServerMonitor.prototype.notifyObservers = function(json) {
	var increaseTimeout = false;

	this.listeners.forEach(function(item) {
		if(item.notify) {
			increaseTimeout = increaseTimeout || item.notify(json);
		}
	});	

	if (increaseTimeout) {
		this.log("Detectado cambio. Reseteando secuencia fibonacci");
		this.start();
	}
}

Nire.ServerMonitor.prototype.executeOnDemand = function(json) {
	if(!json) this._timerCallback();
	else this.notifyObservers(json);
}
