/**
 * Funciones para facilitar el Logging y el desarrollo de la aplicacion
 */
(function (global) {
    var LOGGER_MATCHERS = [];

    function saveMatchers() { 
        var map = LOGGER_MATCHERS.map(function(item) { return item.source });
        sessionStorage.setItem('LOGGER_MATCHERS', map); 
    }

    function restoreMatchers() { 
        var str = sessionStorage.getItem('LOGGER_MATCHERS');
        var strArray = str ? str.split(",") : [];

        LOGGER_MATCHERS = strArray.map(function(item) { return new RegExp(item) }); 
    }

    function addLoggerMatcher(regexp) {
        LOGGER_MATCHERS.push(new RegExp(regexp));
        saveMatchers();
    }

    function clearLoggerMatcher() {
        LOGGER_MATCHERS = [];
        saveMatchers();
    }

    /**
     * Crear un logger para un detemrinado servicio
     * El servicio es únicamente un prefijo que se agregará a los mensajes de log.
     *
     * El loger se limita a enviar a log todos los parámetros recibidos
     */
    restoreMatchers();
    console.log([
        "========================================================================================================================",
        "Si se utiliza el servicio getLogger, se pueden filtrar los logs a mostrar agregando REGEXP de los servicios que se" ,
        "quieren habilitar. Por favor, utilice esta facilidad para evitar contaminar con muchos logs innecesarios para otros la",
        "consola.",
        "",
        "Por ejemplo para mostrar los mensajes de ServerMonitor se debe ejecutar: Logger.add(/ServerMonitor/) en la",
        "consola de Chrome o Firefox",
        "========================================================================================================================"
    ].join("\n"))
    console.log("LOGGERS habilitados ", LOGGER_MATCHERS);

    function getLogger(service) {
        return function() {
            // Si el servicio no se corresponde con ningun matcher no lo mostramos
            var match = LOGGER_MATCHERS.some(function(match) { return match.test(service) });
            if(!match) return;

            var timestamp = new Date().getTime();
            var data = [timestamp, service].concat(Array.prototype.slice.call(arguments));

            // Utilizando adb solo se puede ver el primer parámetro de console.log
            // Si detectamos que estamos en un ambiente Cordova, utilizamos este Log
            if(typeof device !== 'undefined') {
                var forConsole = data.join(" - ");
                console.log(forConsole);
            }

            console.log.apply(console, data);
        }
    }

    //
    // Exportamos los simbolos que nos interesan.
    //
    global.getLogger = getLogger;    
    global.Logger = {
        add: addLoggerMatcher,
        clear: clearLoggerMatcher,
        getLogger: getLogger
    };

}(this));