var Nire = typeof Nire === "undefined" ? {} : Nire;

Nire.ProfileQuestions = function(searchService, params) {
    var defaults = {
        labelPlaceholder: "#questionLabel",
        bodyPlaceholder: "#questionBody",
        onChangeCallbacks: [],
        onShowCallbacks: [],

        templates: {
            text: Handlebars.compile('<input type="text" />'),
            checkbox: Handlebars.compile('{{#each options}}<label><input type="checkbox" value="{{ this.value }}" /> {{ this.label }}</label>{{/each}}'),
            option: Handlebars.compile('{{#each options}}<label><input type="radio" value="{{ this.value }}" /> {{ this.label }}</label>{{/each}}'),
            elasticsearch: Handlebars.compile('<div id="selection"></div><input type="text" /><div id="results"></div>'),
            elasticsearch_result: Handlebars.compile('<div class="result">{{ nombredieta }}</div>'),
            elasticsearch_selection: Handlebars.compile('<div class="item">{{ nombredieta }}</div>')
        }
    }

    this.searchService = searchService;
    this.params = $.extend(true, {}, defaults, params);
}

Nire.ProfileQuestions.prototype.load = function(questions) {
    this.questions = questions;
    this.currentQuestionIndex = -1;
}

Nire.ProfileQuestions.prototype.showNextQuestion = function() {
    this.currentQuestionIndex++;
    var question = this.questions[this.currentQuestionIndex];

    if( this._questionConditionsMet(question) ) {
        this.show(question);
        return question;
    }
    else {
        return this.showNextQuestion();
    }
}

/**
 * Registrar callbacks del evento onChange
 */
Nire.ProfileQuestions.prototype.onChange = function(callback) {
    this.params.onChangeCallbacks.push(callback);
}

/**
 * Registrar callback de evento cuando se muestra una pregunta nueva.
 */
Nire.ProfileQuestions.prototype.onShow = function(callback) {
    this.params.onShowCallbacks.push(callback);    
}

Nire.ProfileQuestions.prototype.show = function(question) {
    this.currentQuestion = question;
    this._updateView();
}

/**
 * Valida la respuesta actual. Y retorna true si es una respuesta aceptable
 */
Nire.ProfileQuestions.prototype.isValid = function(question) {
    // las preguntas tipo checkbox siempre son validas
    if(this.currentQuestion.type === "checkbox") return true;

    // las respuestas del tipo elasticsearch siempre son validas
    if(this.currentQuestion.type === "elasticsearch") return true;

    // Verificamos si es una pregunta obligatoria.
    if(this.currentQuestion.mandatory) {
        if(typeof this.currentQuestion.value === 'undefined') return false;

        if(Array.isArray(this.currentQuestion.value)) {
            // si es un array y no hay nada seleccionado es invalida.
            if(this.currentQuestion.value.length == 0) return false;
        }
        else {
            // Si es un string y no hay escrito nada es invalida.
            if(this.currentQuestion.value.trim() === '') return false;
        }
    }

    return true;
}

Nire.ProfileQuestions.prototype.search = function(text) {
    var self = this;

    if(this.delayTimer) clearTimeout(this.delayTimer);
    if(text.length < 1) return self.showSearchResults([]);

    this.delayTimer = setTimeout(function() {
        var encodedQuery = self.currentQuestion.queryTemplate.replace("#term#", encodeURIComponent(text) + "*");
        self.searchService.search(self.currentQuestion.index, encodedQuery)
            .done( self.showSearchResults.bind(self) );
    }, 250);
}

Nire.ProfileQuestions.prototype._questionConditionsMet = function(question) {
    if(typeof question === 'undefined') return true;
    if(question.conditionQuestion) {
        var refQuestion = this._searchQuestionById(question.conditionQuestion);
        if(typeof refQuestion === 'undefined' || !refQuestion) return true;
        return refQuestion.value == question.conditionValue;
    }

    return true;
}

Nire.ProfileQuestions.prototype._searchQuestionById = function(id) {
    var questions = this.questions.filter(function(item) { return item.id == id });
    if(questions.length > 0) return questions[0];
    return null;
}

/**
 * Usado internamente para notificar de onchanges
 */
Nire.ProfileQuestions.prototype._emitOnChange = function() {
    this.params.onChangeCallbacks.forEach(function(item) { item(); })
}

/**
 * Usado internamente para notificar que se mostro una pregunta
 */
Nire.ProfileQuestions.prototype._emitOnShow = function() {
    this.params.onShowCallbacks.forEach(function(item) { item(); })
}

/**
 * Redibuja la vista para corresponder con el estado de la pregunta actual
 */
Nire.ProfileQuestions.prototype._updateView = function() {
    var self = this;

    if(typeof this.currentQuestion === 'undefined') {
        $(this.params.labelPlaceholder).empty();
        $(this.params.bodyPlaceholder).empty();
    }
    else {
        var template = this.params.templates[this.currentQuestion.type];
        var dom = $(template(this.currentQuestion));

        $(this.params.labelPlaceholder).text(this.currentQuestion.label);
        $(this.params.bodyPlaceholder).html(dom);

        if(this.currentQuestion.type === 'elasticsearch') {
            dom.off(".profile_questions")
                // Si NO cancelamos el evento submit del formulario necesitamos activar este evento
                // .on("keydown.profile_questions", function(e) {
                //     if (e.which === $.ui.keyCode.ENTER) return false; // ignoramos ENTER
                // })
                .on("input.profile_questions", function(e) {
                    self.search( $(this).val() );
                });   
            this.updateSearchResults();
            this.updateSelection();
        }
        else {
            if(this.currentQuestion.value) {
                if(Array.isArray(this.currentQuestion.value)) {
                    this.currentQuestion.value.forEach(function(item) {
                        var selector = "input[value='" + item +"']"
                        $(dom).find(selector).addBack(selector).prop("checked", true);
                    });
                }
                else {
                    var selector = "input[type=radio][value='" + this.currentQuestion.value + "']";
                    $(dom).find(selector).addBack(selector).prop("checked", true);

                    $(dom).find("input[type=text]").addBack("input").val( this.currentQuestion.value );
                }
            }            
        }

        if(self.currentQuestion.type !== 'elasticsearch') {
            dom.off(".profile_questions").on("input.profile_questions change.profile_questions click.profile_questions", dom, function() {
                if(self.currentQuestion.type === 'text')
                    self.currentQuestion.value = $(self.params.bodyPlaceholder).find("input").val();
                else if(self.currentQuestion.type === 'option')
                    self.currentQuestion.value = $(self.params.bodyPlaceholder).find("input:checked").val();
                else if(self.currentQuestion.type === 'checkbox') {
                    setTimeout(function() {
                        self.currentQuestion.value = [];
                        $(self.params.bodyPlaceholder).find("input:checked").each(function() { 
                            self.currentQuestion.value.push(this.value);
                        });
                    }, 0)
                }

                self._emitOnChange();
            });
        };

        this._emitOnShow();
    }
}

// ========================================== //
// == METODOD EXCLUSIVOS DE ELASTIC-SAERCH == //
// ========================================== //

Nire.ProfileQuestions.prototype.showSearchResults = function(data) {
    this.currentQuestion.searchResults = data.map(function(item) { return item._source; });
    this.updateSearchResults();
}

Nire.ProfileQuestions.prototype.selectItem = function(item) {
    if(typeof this.currentQuestion.value === 'undefined')
        this.currentQuestion.value = [];

    this.currentQuestion.value.push(item)
    this.updateSelection();

    this._emitOnChange();
}

Nire.ProfileQuestions.prototype.removeItem = function(item) {
    if(typeof this.currentQuestion.value === 'undefined')
        this.currentQuestion.value = [];

    var index = this.currentQuestion.value.indexOf(item);
    if(index >= 0) {
        this.currentQuestion.value.splice(index, 1)
        this._updateView();

        this._emitOnChange();
    }
}

Nire.ProfileQuestions.prototype.updateSelection = function() {
    if(typeof this.currentQuestion.value === 'undefined') return

    var self = this;
    var dom = [];

    // TODO: [2015/10/7] Quitar este fragmento de codigo cuando el servidor retorne
    // siempre un valor válido. Esto fue agregado cuando se implementó el tipo de pregunta
    // ElasticSearch
    if(!Array.isArray(this.currentQuestion.value))
        this.currentQuestion.value = [];

    this.currentQuestion.value.forEach(function(item) { 
        var template = self.params.templates["elasticsearch_selection"];
        var currentElement = $(template(item));
        currentElement.on("click", function() { self.removeItem(item); });
            
        dom.push( currentElement );
    });

    $(this.params.bodyPlaceholder).find("#selection").html(dom);
}

Nire.ProfileQuestions.prototype.updateSearchResults = function() {
    if(typeof this.currentQuestion.searchResults === 'undefined') return;
    var self = this;

    var dom = []
    this.currentQuestion.searchResults.forEach(function(item) {
        var template = self.params.templates["elasticsearch_result"];

        var currentElement = $(template(item));
        currentElement.on("click", function() { self.selectItem(item) });
        dom.push(currentElement);
    });

    $(this.params.bodyPlaceholder).find("#results").html(dom)
}