var Nire = typeof Nire === "undefined" ? {} : Nire;

/**
 * Objeto para realizar búsquedas de diferentes items.
 * Principalmente como interfaz contra ElasticSearch
 *
 * @param locale idioma en el cual se supone se realizan las búsquedas.
 */
Nire.Search = function(locale, errorHandler) {
    this.log = getLogger("Search");
    this.locale = locale.toLowerCase().replace("-", "_");
    this.errorHandler = errorHandler || jQuery.noop;
};

/**
 * Realiza una query a travez del SearchAPI
 */
Nire.Search.prototype.search = function(index, query) {
    var self = this;

    var url = URL_ELASTICSEARCH + index + "/_search?q=" + query;
    return $.get(url)
        .fail( this.errorHandler )
        .then(function(data) {
            self.log("Resultado ElasticSearch", data);
            return data.hits.hits
        });
}

/**
 * Busca un plato de acuerdo a su nombre
 * @param query palabras o letras a  buscar en el nombre del plato
 */
Nire.Search.prototype.plateByName = function(query) {
    // Si el query es una letra o menos, retornamos vacio
    if(query.length < 2) {
        var result = $.Deferred();
        result.resolve([]);
        return result;
    }

    var encodedQuery = "nombredieta:" + encodeURIComponent(query);
    return this.search("platos_" + this.locale, encodedQuery)
        .then(function(hits) {
            return hits.map(function(item) {
                return { 
                    name: item._source.nombredieta, 
                    id: item._source.id,
                    kcal: item._source.kcal
                };
            });
        });
};
