describe("Interacciones con el la página HOST", function() {
    var sut;
    var questions

    beforeEach(function() {
        $("#scrap").html("<h1 id='title'></h1><div id='body'></div>")

        var deferred = $.Deferred();
        deferred.resolve([{ _source: { label: 'result 1'} }, {_source: { label: 'result 2'}}, {_source: { label: 'result 3'}}]);
        searchService = { search: function(index, query) { return deferred.promise(); } };

        questions = [
            { id: "question1", label: "First question", type: "text" },
            { id: "question2", label: "Second question", type: "option", options: [{ label: 'opcion 1', value: 'opc1' }, { label: 'opcion 2', value: 'opc2' }] },
            { id: "question3", label: "Third question", type: "checkbox", options: [{ label: 'opcion 1', value: 'opc1' }, { label: 'opcion 2', value: 'opc2' }]  },
            { id: "question4", label: "Fourth question", type: "elasticsearch" },
        ];

        sut = new Nire.ProfileQuestions(searchService, {
            labelPlaceholder: "#title",
            bodyPlaceholder: "#body",
        });
    }); 

    describe("Notificaciones cuando se cambia el valor", function() {
        var callback

        beforeEach(function() {
            callback = sinon.spy()
            sut.onChange( callback );
        });

        it("preguntas del tipo text", function() {
            sut.show(questions[0]);
            $("#body input").val("new value").trigger("input");

            expect(callback.callCount).to.be.equals(1);
        });

        it("preguntas del tipo option", function() {
            sut.show(questions[1]);

            $($("#body input")[1])
                .prop("checked", true)
                .change();

            expect(callback.callCount).to.be.equals(1);
        });

        it("preguntas del tipo checkbox", function() {
            sut.show(questions[2]);

            $($("#body input")[1])
                .prop("checked", true)
                .change();

            expect(callback.callCount).to.be.equals(1);
        });

        describe("preguntas del tipo elasticsearch", function() {
            it("el input de busqueda no debe generar un evento de cambio", function() {
                sut.show(questions[3]);

                $($("#body input")[1])
                    .val("busqueda")
                    .keypress();

                expect(callback.callCount).to.be.equals(0);
            });

            it("el seleccionar una opcion genera un evento de cambio", function() {
                sut.show(questions[3]);
                sut.selectItem({ label: "opcion" })

                expect(callback.callCount).to.be.equals(1);
            });

            it("quitar una opcion genera un evento de cambio", function() {
                var opcionElegida = { label: "opcion"};

                questions[3].value = [opcionElegida];
                sut.show(questions[3]);
                sut.removeItem(opcionElegida)

                expect(callback.callCount).to.be.equals(1);
            });
        });
    });

    it("Notificaciones cuando se muestra una pregunta", function() {
        var callback = sinon.spy();
        sut.onShow( callback );

        sut.show(questions[0]);
        expect(callback.callCount).to.be.equals(1);
    });

    describe("Validez de las preguntas", function() {
        it("las preguntas no obligatorias siempre son validas", function() {
            sut.show(questions[0]);
            expect(sut.isValid()).to.be.true;
        });

        it("las preguntas de texto requeridas no pueden estar vacias", function() {
            questions[0].mandatory = true;
            sut.show(questions[0]);

            // Antes de escribir algo es invalida
            expect(sut.isValid()).to.be.false;

            // Despues de escribir algo es valida
            $("#body input").val("new value").trigger("input");
            expect(sut.isValid()).to.be.true;

            // Si esta vacio es invalida
            $("#body input").val("").trigger("input");
            expect(sut.isValid()).to.be.false;
        });    

        it("las preguntas checkbox incluso siendo obligatorias pueden estar vacias", function() {
            questions[2].mandatory = true;
            sut.show(questions[2]);

            // Si no hay nada seleccionado es valida
            expect(sut.isValid()).to.be.true;

            // Si hay algo seleccionado tambien es valida
            $($("#body input")[1]).prop("checked", true).change();
            expect(sut.isValid()).to.be.true;
        });

        it("las preguntas de eleasticsearch incluso siendo obligatorias pueden estar vacias", function() {
            questions[3].mandatory = true;
            sut.show(questions[3]);

            // Si no hay nada seleccionado es valida
            expect(sut.isValid()).to.be.true;

            // Si hay algo seleccionado tambien es valida
            sut.selectItem({ label: "opcion" })
            expect(sut.isValid()).to.be.true;
        });
    })

});