describe('El sevidor puede habilitar o deshabilitar secciones', function() {
    var connection = null;

    beforeEach(function() {
        KNOWN_SECTIONS = ["PHYSICAL", "NUTRITION", "HEALTH", "TREATMENT", "CLINIC"];
        DISABLED_SECTIONS = [];
        connection = new Nire.Connection();
    });

    describe("Configuracion y reconfiguracion", function() {
        beforeEach(function() {
            KNOWN_SECTIONS = ["PHYSICAL", "NUTRITION", "HEALTH", "TREATMENT", "CLINIC"];
            DISABLED_SECTIONS = ["CLINIC"];
            connection = new Nire.Connection();
        });

        it("Si no se configura nada se utilizan los valores defaults", function() {
            var result = connection.getEnabledSections();
            assert.sameMembers(result, ["PHYSICAL", "NUTRITION", "HEALTH", "TREATMENT"]);
        });

        it("Se pueden cambiar las secciones habilitadas y deshabilitadas", function() {
            connection.effectiveEnabledSections(["CLINIC"], ["NUTRITION"])

            var result = connection.getEnabledSections();
            assert.sameMembers(result, ["PHYSICAL", "HEALTH", "TREATMENT", "CLINIC"]);
        });

        it("Si las secciones habilitadas cambian se notifica a un callback", function() {
            var callback = sinon.spy();
            
            connection.onSectionChange( callback );
            connection.effectiveEnabledSections(["CLINIC"]);

            assert.ok( callback.calledOnce );
        });

        it("Si no hay ningun cambio real no se notifica al callback", function() {
            var callback = sinon.spy();
            
            // Notar que Nutrition ya estaba habilitada
            connection.onSectionChange( callback );
            connection.effectiveEnabledSections(["NUTRITION"]);

            assert.ok( callback.notCalled );
        })
    })

    describe('Effective Enabled Sections', function () {
        it('se conocen todas las secciones', function () {
            assert.ok( KNOWN_SECTIONS );
        });
        it('allow * retorna toda las secciones conocidas', function() {
            assert.deepEqual(connection.effectiveEnabledSections(['*']), KNOWN_SECTIONS);
        });
        it('las secciones listadas en deny no son retornadas', function() {
            assert.deepEqual(connection.effectiveEnabledSections(['*'], ['PHYSICAL', 'NUTRITION']), ['HEALTH', 'TREATMENT', 'CLINIC']);
        });
        it('las secciones que por default estan negadas no se muestran', function() {
            DISABLED_SECTIONS = ["TREATMENT"];
            assert.deepEqual(connection.effectiveEnabledSections(), ['PHYSICAL', 'NUTRITION', 'HEALTH', 'CLINIC']);
        });
        it('si una seccion es permitida explicitamente se muestra aunque su default se negada', function() {
            assert.deepEqual(connection.effectiveEnabledSections(['TREATMENT']), ['PHYSICAL', 'NUTRITION', 'HEALTH', 'TREATMENT', 'CLINIC']);
        });
        it('tiene prioridad el deny', function() {
            assert.deepEqual(connection.effectiveEnabledSections(['TREATMENT'], ['TREATMENT']), ['PHYSICAL', 'NUTRITION', 'HEALTH', 'CLINIC']);
        });
    });

    describe('Enabling/Dissabling Sections', function() {
        before(function() {
            $("#scrap").html('<div id="physical" class="SECTION_PHYSICAL">physical</div><div id="nutrition" class="SECTION_NUTRITION">nutrition</div>')
        });

        beforeEach(function() {
            $("#__HIDDEN_SECTIONS_CSS__").remove();
        });

        it('las secciones enumeradas son mostradas', function() {
            connection.configureEnabledSections(['PHYSICAL', 'NUTRITION'])

            assert.ok( $("#physical").is(":visible") );
            assert.ok( $("#nutrition").is(":visible") );
        });
        it('las secciones no enumeradas se ocultan', function() {
            connection.configureEnabledSections(['PHYSICAL'])

            assert.ok( $("#physical").is(":visible") );
            assert.notOk( $("#nutrition").is(":visible") );
        });
        it('se pueden modificar la lista de secciones habilitadas', function() {
            connection.configureEnabledSections(['PHYSICAL'])
            connection.configureEnabledSections(['NUTRITION'])

            assert.notOk( $("#physical").is(":visible") );
            assert.ok( $("#nutrition").is(":visible") );
        });
    });

    describe("isEnabledSection", function() {
        it("retorna true para las secciones habilitadas", function() {
            DISABLED_SECTIONS = ["CLINIC"];
            connection.configureEnabledSections(
                connection.effectiveEnabledSections(KNOWN_SECTIONS, DISABLED_SECTIONS)
            );            
            assert.ok(connection.isEnabledSection('NUTRITION'));
        });
        it("retorna false para las secciones deshabilitadas", function() {
            DISABLED_SECTIONS = ["CLINIC"];
            connection.configureEnabledSections(
                connection.effectiveEnabledSections(KNOWN_SECTIONS, DISABLED_SECTIONS)
            );            
            assert.notOk(connection.isEnabledSection('CLINIC'));
        })
    });
});
