describe("PhysicalActivitySteps", function() {
    var IMAGE = "https://www.google.com.ar/images/srpr/logo11w.png";
    var NO_IMAGE = "https://www.google.com.ar/do_not_exists.png";
    var PLACEHOLDER_IMAGE = "../www/img/others/img_placeholder.png";

    before(function() {
        // Mockeamos TryImage para no depender de datos externos.
        // El funcionamiento de TryImage se testea en otra suite.
        sinon.stub(window, "tryImage", function(image) {
            var deferred = $.Deferred();

            if(image == IMAGE) deferred.resolve(IMAGE);
            else deferred.reject();

            return deferred.promise();        
        });
    });

    after(function() {
        window.tryImage.restore();
    });

    beforeEach(function() {
        $("#scrap").html('<div id="page_physical_activity_steps"><img src="../www/img/others/img_placeholder.png" class="img_image"/></div>');
    });

    function replaceImageGivenExcerciseAndStep(excercise, step) {
        var json = [{image: step}]

        pagePhysicalActivitySteps.$page = $("#page_physical_activity_steps");
        pagePhysicalActivitySteps.exerciseImage = excercise;
        pagePhysicalActivitySteps.displayStep(json, 0);
        return $("#scrap .img_image").attr("src");
    }

    it("si no se hace nada se muestra el placeholder", function() {
        assert.equal( $("#scrap .img_image").attr("src"), PLACEHOLDER_IMAGE );
    });
    it("si se logra cargar la imagen del step la muestra", function() {
        assert.equal(replaceImageGivenExcerciseAndStep(NO_IMAGE, IMAGE), IMAGE);
    });
    it("si el step no tiene imagen se muestra la imagen del ejercicio", function() {
        assert.equal(replaceImageGivenExcerciseAndStep(IMAGE, ""), IMAGE);
    });
    it("si no se puede cargar la imagen del step se muestra la del ejercicio", function() {
        assert.equal(replaceImageGivenExcerciseAndStep(IMAGE, NO_IMAGE), IMAGE);
    });
    it("si no se puede cargar el ejercicio ni el step se sigue mostrando el placeholder", function() {
        assert.equal(replaceImageGivenExcerciseAndStep(NO_IMAGE, NO_IMAGE), PLACEHOLDER_IMAGE);
    });
});